// *************************************************************************************
// Project:  LED Anzeige mit ATmega48A
// Software: AVR-Studio v4.19 / WinAVR-20100110 / avr-toolchain-installer-3.4.2.1573
// Edit:     Alexander Kilpert / Jens Grtler
// Erstellt: 18.11.2017
// *************************************************************************************

// --- einbinden von Libs --------------------------------------------------------------
#include <avr/io.h>                          // AVR Basisdaten einbinden
#include <avr/interrupt.h>                   // Interrupts bereitstellen
#include <avr/wdt.h>                         // Watchdog Routine einbinden

// --- Makros ------------------------------------------------
#define Setzen(adr,bit)   (adr |= (1<<bit))  // Bit setzen
#define Loeschen(adr,bit) (adr &= ~(1<<bit)) // Bit lschen
#define Toggeln(adr,bit)  (adr ^= (1<<bit))  // Bit Toogeln
#define High(adr,bit)     (adr & (1<<bit))   // gesetzes Bit in Adresse
#define Low(adr,bit)    (!(adr & (1<<bit)))  // gesetzes Bit in Adresse

// --- Definiton der Konstanten --------------------------------------------------------
#define T1_vorladen      63036               // Timer1 vorladen (20ms)
#define I2C_Adr          20                  // I2C-Bus Ardesse
#define LED_BLF          15                  // Blinkfrequenz LEDs (15ms x 20ms = 300ms)
#define AL_Interval      50                  // Alarm Intervall (50ms x 20ms = 1Sek)

// --- Definiton der Globalen Variabeln ------------------------------------------------
volatile uint8_t Daten01 = 0;                // I2C-Bus Daten-Byte Nr.1
         // Bit0 = LED Vdd Kanal1 (0=OFF, 1=ON)
         // Bit1 = LED Vdd Kanal1 (1=Blinken)
         // Bit2 = LED K1 Tacho1 (0=OFF, 1=ON)
         // Bit3 = LED K1 Tacho1 (1=Blinken)
         // Bit4 = LED K1 Tacho2 (0=OFF, 1=ON)
         // Bit5 = LED K1 Tacho2 (1=Blinken)
         // Bit6 = LED K1 Tacho3 (0=OFF, 1=ON)
         // Bit7 = LED K1 Tacho3 (1=Blinken)
volatile uint8_t Daten02 = 0;                // I2C-Bus Daten-Byte Nr.2
         // Bit0 = LED Vdd Kanal2 (0=OFF, 1=ON)
         // Bit1 = LED Vdd Kanal2 (1=Blinken)
         // Bit2 = LED K2 Tacho1 (0=OFF, 1=ON)
         // Bit3 = LED K2 Tacho1 (1=Blinken)
         // Bit4 = LED K2 Tacho2 (0=OFF, 1=ON)
         // Bit5 = LED K2 Tacho2 (1=Blinken)
         // Bit6 = LED K2 Tacho3 (0=OFF, 1=ON)
         // Bit7 = LED K2 Tacho3 (1=Blinken)
volatile uint8_t Daten03 = 0;                // I2C-Bus Daten-Byte Nr.3
         // Bit0 = LED DFM (0=OFF, 1=ON)
         // Bit1 = LED DFM (1=Blinken)
         // Bit2 = LED Alarm (0=OFF, 1=ON) (Hauptalarm ON)
         // Bit3 = LED Alarm (1=Blinken) (Voralarm ON)
         // Bit4 = Piezo Voralarm (0=OFF, 1=Intervall)
         // Bit5 = Piezo Hauptalarm (0=OFF, 1=Intervall)
         // Bit6 = Piezo System ON/OFF (0=OFF, 1=Intervall)
         // Bit7 = frei
volatile uint8_t Flag01 = 0;                 // Flag Nr.1
         // Bit0 = Blinkfrequenz 0,3 Sek
         // Bit1 = Intervallzeit 1 Sek
         // Bit2 = frei
         // Bit3 = frei
         // Bit4 = Verriegelung Voralarm
         // Bit5 = Verriegelung Hauptalarm
         // Bit6 = Verriegelung Piezo System ON/OFF
         // Bit7 = Taster entprellt (1=bettigt)
volatile uint8_t I2C_Zeiger = 0;             // Zeiger fr I2C-Bus

// --- einbinden externer Dateien ----------------------------
#include "Pause.h"                           // Pause Routinen einbinden


// *************************************************************************************
// *   INTERRUPT Vektor 14 - Timer1 berlauf (alle 20ms)                               *
// *************************************************************************************
ISR(TIMER1_OVF_vect) {                       // Beginn Timer1 berlauf
   static unsigned char Zeit01;              // Zeit-Zhler Nr.1
   static unsigned char Zeit02;              // Zeit-Zhler Nr.2
   TCNT1 = T1_vorladen;                      // Timer1 neu vorladen
   Zeit01++;                                 // Zeit-Zhler hoch zhlen
   Zeit02++;                                 // Zeit-Zhler hoch zhlen
   // nach 20ms
   if High(PINB,PINB6) { Loeschen(Flag01,7); }  // wenn Taster offen, dann Bit7 lschen
   if Low(PINB,PINB6) { Setzen(Flag01,7); }  // wenn Taster bettigt, dann Bit7 setzen
   // nach 300ms
   if (Zeit01 > LED_BLF) {                   // wenn 300ms vergangen sind, dann...
      Zeit01 = 0;                            // Reset
      Toggeln(Flag01,0);                     // Bit0 toggeln
   }
   // nach 1 Sekunde
   if (Zeit02 > AL_Interval) {               // wenn 1000ms vergangen sind, dann...
      Zeit02 = 0;                            // Reset
      Toggeln(Flag01,1);                     // Bit1 toggeln
   }
}                                            // Ende Timer1 berlauf

// *************************************************************************************
// *   INTERRUPT Vektor 25 - TWI Interrupt auswerten                                   *
// *************************************************************************************
ISR(TWI_vect) {                              // Beginn I2C-Interrupt
   // Daten empfangen
   if (TWSR == 0x60) {                       // wenn Slave-W-Adresse erkannt wurde, dann...
      I2C_Zeiger = 0;                        // Reset - Zeiger zurck setzen
   }                                         // Ende wenn Slave-W-Adresse erkannt wurde
   if (TWSR == 0x80) {                       // wenn Daten empfangen werden, dann...
      if (I2C_Zeiger == 0) { Daten01 = TWDR; }  // Daten an Byte1 bergeben
      if (I2C_Zeiger == 1) { Daten02 = TWDR; }  // Daten an Byte2 bergeben
      if (I2C_Zeiger == 2) { Daten03 = TWDR; }  // Daten an Byte3 bergeben
      I2C_Zeiger++;                          // Zeiger erhhen
   }                                         // Ende wenn Daten empfangen werden
   // Daten senden
   if (TWSR == 0xA8) {                       // wenn Slave-R-Adresse erkannt wurde, dann...
      I2C_Zeiger = 0;                        // Reset - Zeiger zurck setzen
      TWDR = Flag01;                         // Flag - Taster an I2C-Bus bergeben
   }                                         // Ende wenn Slave-R-Adresse erkannt wurde
//   if (TWSR == 0xB8) {                       // wenn Read-ACK erkannt wurde, dann...
//      I2C_Zeiger++;                          // Zeiger erhhen
//      if (I2C_Zeiger == 1) { TWDR = Relais_Port2; }  // Byte2 an I2C-Bus bergeben
//      if (I2C_Zeiger == 2) { TWDR = Relais_Port3; }  // Byte3 an I2C-Bus bergeben
//   }                                         // Ende wenn Read-ACK erkannt wurde
   // I2C-Stop
// if ((TWSR == 0xA0) || (TWSR == 0xC0)) { PORTD &= ~(1<<PD3); } // wenn I2C-Stop erkannt wurde, dann LED AUSschalten
   // weiter im I2C-Bus...
   TWCR |= (1<<TWINT);                       // TWINT setzen - TWINT lschen, weiter im I2C-Bus...
}

// *************************************************************************************
// *   FUNKTION - LED & Piezp Start-Test                                               *
// *************************************************************************************
void Start_Test(void) {                      // Beginn Start-Test
   // --- Kanal Nr.1 --- ON ---
   Setzen(PORTB,PB0);                        // Vdd LED ON
   Setzen(PORTD,PD7);                        // Tacho1 LED ON
   Setzen(PORTD,PD6);                        // Tacho2 LED ON
   Setzen(PORTD,PD5);                        // Tacho3 LED ON
   // --- Kanal Nr.2 --- ON ---
   Setzen(PORTB,PB1);                        // Vdd LED ON
   Setzen(PORTB,PB2);                        // Tacho1 LED ON
   Setzen(PORTC,PC0);                        // Tacho2 LED ON
   Setzen(PORTC,PC1);                        // Tacho3 LED ON
   // --- DFM --- ON ---
   Setzen(PORTC,PC2);                        // DFM LED ON
   // --- Alarm --- ON ---
   Setzen(PORTC,PC3);                        // Alarm LED ON
   // --- Piezo --- ON ---
   Setzen(PORTB,PB7);                        // Piezo ON
   Pause(1000);                              // 1 Sek warten
   // --- Piezo --- OFF ---
   Loeschen(PORTB,PB7);                      // Piezo OFF
   Pause(2000);                              // 2 Sek warten
   // --- Kanal Nr.1 --- OFF ---
   Loeschen(PORTB,PB0);                      // Vdd LED OFF
   Loeschen(PORTD,PD7);                      // Tacho1 LED OFF
   Loeschen(PORTD,PD6);                      // Tacho2 LED OFF
   Loeschen(PORTD,PD5);                      // Tacho3 LED OFF
   // --- Kanal Nr.2 --- OFF ---
   Loeschen(PORTB,PB1);                      // Vdd LED OFF
   Loeschen(PORTB,PB2);                      // Tacho1 LED OFF
   Loeschen(PORTC,PC0);                      // Tacho2 LED OFF
   Loeschen(PORTC,PC1);                      // Tacho3 LED OFF
   // --- DFM --- OFF ---
   Loeschen(PORTC,PC2);                      // DFM LED OFF
   // --- Alarm --- OFF ---
   Loeschen(PORTC,PC3);                      // Alarm LED OFF
}                                            // Ende Start-Test


// *************************************************************************************
// *   Hauptprogramm                                                                   *
// *************************************************************************************
int main(void) {                             // Beginn Hauptprogramm

   // --- Watchdog abschalten (nach einen Reset) ---------------------------------------
   MCUSR &= ~(1<<WDRF);                      // Watchdog OFF
   WDTCSR |= (1<<WDCE) | (1<<WDE);           // Watchdog OFF
   WDTCSR = 0x00;                            // Watchdog OFF

   // --- Definiton der Lokalen Variabeln ----------------------------------------------
   uint8_t  Piezo = 0;                       // Piezo ON/OFF

   // --- Eingnge setzen --------------------------------------------------------------
   // PB3 - MOSI - frei
   // PB4 - MISO - frei
   // PB5 - SCK - frei
   // PB6 - Taster
   DDRB &= ~((1<<DDB3) | (1<<DDB4) | (1<<DDB5) | (1<<DDB6));  // B-Register - als Eingang setzen
   PORTB |= (1<<PB3) | (1<<PB4) | (1<<PB5) | (1<<PB6);        // Interne Pull-Up fr alle setzen (1)
   // PD0 - frei
   // PD1 - frei
   // PD2 - frei
   // PD3 - frei
   // PD4 - frei
   DDRD &= ~((1<<DDD0) | (1<<DDD1) | (1<<DDD2) | (1<<DDD3) | (1<<DDD4));  // D-Register - als Eingang setzen
   PORTD |= (1<<PD0) | (1<<PD1) | (1<<PD2) | (1<<PD3) | (1<<PD4);         // Interne Pull-Up setzen (1)

   // --- Ausgnge setzen --------------------------------------------------------------
   // PB0 - LED Vdd Kanal1
   // PB1 - LED Vdd Kanal2
   // PB2 - LED Kanal2 Tacho1
   // PB7 - Piezo
   DDRB |= (1<<DDB0) | (1<<DDB1) | (1<<DDB2) | (1<<DDB7);   // B-Register - als Ausgang setzen
   PORTB &= ~((1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB7));   // Ports ausschalten (0)
   // PC0 - LED Kanal2 Tacho2
   // PC1 - LED Kanal2 Tacho3
   // PC2 - LED DFM
   // PC3 - LED Alarm
   DDRC |= (1<<DDC0) | (1<<DDC1) | (1<<DDC2) | (1<<DDC3);   // C-Register - als Ausgang setzen
   PORTC &= ~((1<<PC0) | (1<<PC1) | (1<<PC2) | (1<<PC3));   // Ports ausschalten (0)
   // PD5 - LED Kanal1 Tacho3
   // PD6 - LED Kanal1 Tacho2
   // PD7 - LED Kanal1 Tacho1
   DDRD |= (1<<DDD5) | (1<<DDD6) | (1<<DDD7);   // D-Register - als Ausgang setzen
   PORTD &= ~((1<<PD5) | (1<<PD6) | (1<<PD7));  // Ports ausschalten (0)

   // --- TWI-Modul aktivieren ---------------------------------------------------------
   // PC4	SDA - I2C-Bus (wird vom TWI-Modul zugewiesen)
   // PC5	SCL - I2C-Bus (wird vom TWI-Modul zugewiesen)
   // TWI deaktivieren
   TWCR = 0;                                 // TWI deaktivieren
   // Ports als Eingang setzen
   DDRC &= ~( (1<<DDC4) | (1<<DDC5) );       // C-Register - PC4 und PC5 als Eingang setzen
   PORTC &= ~( (1<<PC4) | (1<<PC5) );        // Interne Pull-Up abschalten (0)
   // I2C-Bus Register setzen
   TWAR = I2C_Adr;                           // I2C Slave Adresse festlegen
   TWCR = (1<<TWEN)|(1<<TWEA)|(1<<TWIE);     // TWI Modul aktivieren, ACK aktivieren, Interrupt aktivieren

   // --- Timer1 - 20ms Zeitbasis ------------------------------------------------------
   TIMSK1 |= (1<<TOIE1);                     // Timer1 Overflow Interrupt Enable
   TCNT1 = T1_vorladen;                      // Timer1 vorladen
   TCCR1B |= (1<<CS11) | (1<<CS10);          // Timer1 Vorteiler 64 & aktivieren

   // --- Interrupt aktivieren ---------------------------------------------------------
   sei();                                    // Interrupts aktivieren

   // --- Initialisierung / Voreinstellung ---------------------------------------------
   Start_Test();                             // LEDs & Piezp testen

   // --- Watchdog aktivieren ----------------------------------------------------------
   wdt_enable(WDTO_2S);                      // Watchdog auf 2 Sekunde einstellen

   // **********************************************************************************
   // *   Endlosschleife                                                               *
   // **********************************************************************************
   while(1) {                                // Beginn While - Endlosschleife

      // --- Kanal Nr.1 --- Vdd LED ----------------------------------------------------
      if High(Daten01,0) {                   // wenn LED ON, dann...
         if High(Daten01,1) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTB,PB0); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTB,PB0); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTB,PB0); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTB,PB0); }          // LED OFF

      // --- Kanal Nr.1 --- Tacho Nr.1 -------------------------------------------------
      if High(Daten01,2) {                   // wenn LED ON, dann...
         if High(Daten01,3) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTD,PD7); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTD,PD7); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTD,PD7); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTD,PD7); }          // LED OFF

      // --- Kanal Nr.1 --- Tacho Nr.2 -------------------------------------------------
      if High(Daten01,4) {                   // wenn LED ON, dann...
         if High(Daten01,5) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTD,PD6); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTD,PD6); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTD,PD6); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTD,PD6); }          // LED OFF

      // --- Kanal Nr.1 --- Tacho Nr.3 -------------------------------------------------
      if High(Daten01,6) {                   // wenn LED ON, dann...
         if High(Daten01,7) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTD,PD5); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTD,PD5); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTD,PD5); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTD,PD5); }          // LED OFF

      // --- Kanal Nr.2 --- Vdd LED ----------------------------------------------------
      if High(Daten02,0) {                   // wenn LED ON, dann...
         if High(Daten02,1) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTB,PB1); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTB,PB1); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTB,PB1); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTB,PB1); }          // LED OFF

      // --- Kanal Nr.2 --- Tacho Nr.1 -------------------------------------------------
      if High(Daten02,2) {                   // wenn LED ON, dann...
         if High(Daten02,3) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTB,PB2); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTB,PB2); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTB,PB2); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTB,PB2); }          // LED OFF

      // --- Kanal Nr.2 --- Tacho Nr.2 -------------------------------------------------
      if High(Daten02,4) {                   // wenn LED ON, dann...
         if High(Daten02,5) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTC,PC0); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTC,PC0); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTC,PC0); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTC,PC0); }          // LED OFF

      // --- Kanal Nr.2 --- Tacho Nr.3 -------------------------------------------------
      if High(Daten02,6) {                   // wenn LED ON, dann...
         if High(Daten02,7) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTC,PC1); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTC,PC1); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTC,PC1); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTC,PC1); }          // LED OFF

      // --- DFM -----------------------------------------------------------------------
      if High(Daten03,0) {                   // wenn LED ON, dann...
         if High(Daten03,1) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTC,PC2); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTC,PC2); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         else { Setzen(PORTC,PC2); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTC,PC2); }          // LED OFF

      // --- Alarm ---------------------------------------------------------------------
      if High(Daten03,2) {                   // wenn LED ON, dann...
         // Voralarm
         if High(Daten03,3) {                // wenn LED TOGLE, dann...
            if High(Flag01,0) { Setzen(PORTC,PC3); }  // wenn Flag ON, dann LED ON
            else { Loeschen(PORTC,PC3); }    // wenn Flag OFF, dann LED OFF
         }                                   // Ende LED TOGLE
         // Hauptalarm
         else { Setzen(PORTC,PC3); }         // LED ON
      }                                      // Ende LED ON
      else { Loeschen(PORTC,PC3); }          // LED OFF

      // --- Piezo ---------------------------------------------------------------------
      // Taster
      if High(Flag01,7) {                    // wenn Taster bettigt, dann...
         if High(Daten03,4) { Setzen(Flag01,4); } // Piezo Voralarm abschalten
         if High(Daten03,5) { Setzen(Flag01,5); } // Piezo Hauptalarm abschalten
         if High(Daten03,6) { Setzen(Flag01,6); } // Piezo System abschalten
      }                                      // Ende Taster bettigt
      // Alarm
      Piezo = 0;                             // Reset Freigabe
      if (High(Daten03,4) && Low(Flag01,4)) { Piezo = 1; } // wenn Piezo Voralarm, dann Freigabe
      if (High(Daten03,5) && Low(Flag01,5)) { Piezo = 1; } // wenn Piezo Hauptalarm, dann Freigabe
      if (High(Daten03,6) && Low(Flag01,6)) { Piezo = 1; } // wenn Piezo System, dann Freigabe
      if (Piezo > 0) {                       // wenn Freigabe, dann...
         if High(Flag01,1) { Setzen(PORTB,PB7); }  // wenn Flag ON, dann Piezo ON
         else { Loeschen(PORTB,PB7); }       // wenn Flag OFF, dann Piezo OFF
      }                                      // Ende Freigabe
      else { Loeschen(PORTB,PB7); }          // wenn keine Freigebe, dann Piezo OFF
      // Reset Verriegelung
      if Low(Daten03,4) { Loeschen(Flag01,4); } // Verriegelung Voralarm aufheben
      if Low(Daten03,5) { Loeschen(Flag01,5); } // Verriegelung Hauptalarm aufhaben
      if Low(Daten03,6) { Loeschen(Flag01,6); } // Verriegelung System aufhaben

      // --- I2C-Bus Reset bei Error ---------------------------------------------------
      if (TWSR == 0x00) {                    // wenn I2C-Bus Error erkannt, dann...
         TWCR |= (1<<TWSTO) | (1<<TWINT);    // TWSTO und TWINT setzen - TWINT lschen, weiter im I2C-Bus...
      }                                      // Ende I2C-Bus Error erkannt

      // --- Hardware Watchdog zurcksetzen --------------------------------------------
      wdt_reset();                           // Watchdog Reset

   }                                         // Ende While Schleife
}                                            // Ende Hauptprogramm






