// ***********************************************************
// Project:  Pause
// Software: AVR-Studio v4.18 / WinAVR-20090313
// Author:   Jens Grtler
// Erstellt: 22.06.2014
// ***********************************************************


// --- einbinden von Libs ------------------------------------
#include <avr\io.h>                          // AVR Basisdaten einbinden
#include <avr/interrupt.h>                   // Interrupts bereitstellen

// --- Definition 1-Wire Daten-Port --------------------------
#define DDR_W            DDRC                // DDR-Register 1-Wire Datenleitung 
#define DD_W             DDC3                // DDR-Pin 1-Wire Datenleitung 
#define PORT_W           PORTC               // PORT-Register 1-Wire Datenleitung 
#define P_W              PC3                 // PORT-Pin 1-Wire Datenleitung 
#define PIN_W            PINC                // Eingang-Pin 1-Wire Datenleitung 

// --- 1-Wire Zeiten mit 18,432MHz Quarz ---------------------
#define Z_480us          1273                // Zeit ca. 480s 
#define Z_60us           160                 // Zeit ca. 60s 
#define Z_420us          1110                // Zeit ca. 420s 
#define Z_1us            2                   // Zeit ca. 1s 
#define Z_14us           37                  // Zeit ca. 14s 
#define Z_45us           119                 // Zeit ca. 45s 

// --- 1-Wire Zeiten mit 16MHz Quarz -------------------------
//#define Z_480us          1100                // Zeit ca. 480s 
//#define Z_60us           140                 // Zeit ca. 60s 
//#define Z_420us          976                 // Zeit ca. 420s 
//#define Z_1us            2                   // Zeit ca. 1s 
//#define Z_14us           32                  // Zeit ca. 14s 
//#define Z_45us           104                 // Zeit ca. 45s 

// --- Definiton der Globalen Variabeln ----------------------
unsigned char OW_Rom[] = {0, 0, 0, 0, 0, 0, 0, 0, 0};  // Array 1-Wire ROM-Daten
unsigned char OW_Data[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};  // Array 1-Wire Scratch-Pad-Daten
unsigned char OW_EEprom[] = {65, 60, 127};   // Array 1-Wire EEprom-Daten


// *************************************************************************************
// *   ROUTINE - 1-Wire Initialisieren                                                 *
// *************************************************************************************
void Init_1_Wire(void) {                     // Beginn 1-Wire Ports einstellen
   // 1-Wire-Datenleitung als Eingang
   DDR_W &= ~(1<<DD_W);                      // Register als Eingang setzen
   PORT_W |= (1<<P_W);                       // Interne Pull-Up setzen (1)
}                                            // Ende 1-Wire Ports einstellen


// *************************************************************************************
// *   ROUTINE - 1-Wire Zeit-Basis                                                     *
// *************************************************************************************
void OW_Zeit(uint16_t ow_zue) {              // Beginn Zeit-Basis
   for(; ow_zue>0; ow_zue--) {               // For a=Zahl To 0 Step -1
      asm volatile("NOP");                   // Ein M Takt nichts tun
   }                                         // Ende FOR
}                                            // Ende Zeit-Basis


// *************************************************************************************
// *   ROUTINE - 1-Wire Reset                                                          *
// *************************************************************************************
unsigned char OW_Reset(void) {               // Beginn Reset
   uint8_t aws = 1;                          // Anwesenheit Slave (1 = keiner da)
   // Interrupt sperren
   cli();                                    // Interrupts deaktivieren
   // 1-Wire-Pin als Ausgang & Lwo setzen
   DDR_W |= (1<<DD_W);                       // als Ausgang setzen
   PORT_W &= ~(1<<P_W);                      // Port ausschalten (0)
   OW_Zeit(Z_480us);                         // mindestens 480s warten
   // 1-Wire-Pin als Eingang (damit High) setzen
   DDR_W &= ~(1<<DD_W);                      // als Eingang setzen
   PORT_W |= (1<<P_W);                       // Interne Pull-Up setzen (1)
   OW_Zeit(Z_60us);                          // mindestens 60s warten
   // Anwesenheit Slave einlesen
   if (!(PIN_W & (1<<P_W))) { aws = 0; }     // wenn Low, dann Slave anwesend
   OW_Zeit(Z_420us);                         // Restzeit von 420s warten
   // Interrupt freigeben
   sei();                                    // Interrupts aktivieren
   return aws;                               // Anwesenheit bergeben
}                                            // Ende Reset


// *************************************************************************************
// *   ROUTINE - 1-Wire Bit senden                                                     *
// *************************************************************************************
void OW_Wr_Bit(uint8_t ow_bit) {             // Beginn Bit senden
   // Interrupt sperren
   cli();                                    // Interrupts deaktivieren
   // 1-Wire-Pin als Ausgang & Lwo setzen
   DDR_W |= (1<<DD_W);                       // als Ausgang setzen
   PORT_W &= ~(1<<P_W);                      // Port ausschalten (0)
   OW_Zeit(Z_1us);                           // mindestens 1s warten
   // Bit Wert abfragen
   if (ow_bit > 0) {                         // wenn Bit 1, dann...
      // 1-Wire-Pin als Eingang (damit High) setzen
      DDR_W &= ~(1<<DD_W);                   // als Eingang setzen
      PORT_W |= (1<<P_W);                    // Interne Pull-Up setzen (1)
   }                                         // Ende Bit 1
   // Ende des Time Slot abwarten
   OW_Zeit(Z_60us);                          // Restzeit von 59s warten
   // 1-Wire-Pin als Eingang (damit High) setzen
   DDR_W &= ~(1<<DD_W);                      // als Eingang setzen
   PORT_W |= (1<<P_W);                       // Interne Pull-Up setzen (1)
   // Interrupt freigeben
   sei();                                    // Interrupts aktivieren
}                                            // Ende Bit senden


// *************************************************************************************
// *   ROUTINE - 1-Wire Bit lesen                                                      *
// *************************************************************************************
unsigned char OW_Rd_Bit(void) {              // Beginn Bit lesen
   uint8_t ow_bit = 1;                       // eingelesendes Bit
   // Interrupt sperren
   cli();                                    // Interrupts deaktivieren
   // 1-Wire-Pin als Ausgang & Lwo setzen
   DDR_W |= (1<<DD_W);                       // als Ausgang setzen
   PORT_W &= ~(1<<P_W);                      // Port ausschalten (0)
   OW_Zeit(Z_1us);                           // 1s warten
   // 1-Wire-Pin als Eingang (damit High) setzen
   DDR_W &= ~(1<<DD_W);                      // als Eingang setzen
   PORT_W |= (1<<P_W);                       // Interne Pull-Up setzen (1)
   OW_Zeit(Z_14us);                          // sptestens nach 15s einlesen (hier 14s)
   // Bit einlesen
   if (!(PIN_W & (1<<P_W))) { ow_bit = 0; }  // wenn Low, dann Bit 0 erkannt
   // Ende des Time Slot abwarten
   OW_Zeit(Z_45us);                          // Restzeit von 45s warten
   // Interrupt freigeben
   sei();                                    // Interrupts aktivieren
   return ow_bit;                            // Bit bergeben
}                                            // Ende Bit lesen


// *************************************************************************************
// *   ROUTINE - 1-Wire Byte senden                                                    *
// *************************************************************************************
void OW_Write(uint8_t ow_byte) {             // Beginn Byte senden
   uint8_t Maske = 1;                        // Maske fr Bit-schieben
   uint8_t z = 0;                            // Variable For Schleife
   // 8-Bits senden
   for(z=0; z<8; z++) {                      // For z=0 To 7 Step +1
      if (ow_byte & Maske) { OW_Wr_Bit(1); } // wenn Bit=1, dann 1 senden
      else { OW_Wr_Bit(0); }                 // wenn Bit=0, dann 0 senden
      Maske = (Maske << 1);                  // Maske *2
      OW_Zeit(Z_1us);                        // Abstand zwischen Time Slot - 1s warten
   }                                         // Ende FOR
}                                            // Ende Byte senden


// *************************************************************************************
// *   ROUTINE - 1-Wire Byte lesen                                                     *
// *************************************************************************************
unsigned char OW_Read(void) {                // Beginn Byte lesen
   uint8_t ow_tmp = 0;                       // Temp Variable
   uint8_t ow_byte = 0;                      // eingelesendes Byte
   uint8_t z = 0;                            // Variable For Schleife
   // 8-Bits lesen
   for(z=0; z<8; z++) {                      // For z=0 To 7 Step +1
      ow_tmp = OW_Rd_Bit();                  // Bit einlesen
      if (ow_tmp > 0) { ow_byte |= (1<<z); } // wenn Bit=1, dann Byte 1 schieben
   }                                         // Ende FOR
   return ow_byte;                           // Byte bergeben
}                                            // Ende Byte lesen


// *************************************************************************************
// *   ROUTINE - 1-Wire Read-ROM (Lesen der 64-Bit-ROM Identifikation)                 *
// *************************************************************************************
unsigned char OW_Read_ROM(void) {            // Beginn Read-ROM
   uint8_t ow_tmp = 0;                       // Temp Variable
   uint8_t z = 0;                            // Variable For Schleife
   uint8_t Er_Flag = 0;                      // Error Flag (>0 = Fehler)
   // Start mit Reset-Impuls
   ow_tmp = OW_Reset();                      // Reset senden
   if (ow_tmp == 0) {                        // wenn Slave vorhanden, dann...
      // Read-ROM Befehl senden
      OW_Write(0x33);                        // Read-ROM Befehl senden
      // Antwort einlesen
      for(z=0; z<8; z++) {                   // For z=0 To 7 Step +1
         OW_Rom[z] = OW_Read();              // 8-Byte einlesen
      }                                      // Ende FOR
      // CRC berprfen
      CRC8_Sp = 0;                           // CRC8 Speicher lschen
      for(z=0; z<7; z++) {                   // For z=0 To 6 Step +1
         CRC8_berechnen(OW_Rom[z]);          // CRC8 berechnen
      }                                      // Ende FOR
      if (CRC8_Sp != OW_Rom[7]) {            // wenn CRC nicht stimmt, dann...
         Er_Flag++;                          // Fehler erfassen
      }                                      // Ende wenn CRC nicht stimmt
   }                                         // Ende wenn Slave vorhanden
   else {                                    // wenn kein Slave, dann...
      Er_Flag++;                             // Fehler erfassen
   }                                         // Ende wenn kein Slave
   // Error Flag auswerten
   if (Er_Flag > 0) {                        // wenn Fehler vorhanden, dann...
      for(z=0; z<8; z++) {                   // For z=0 To 7 Step +1
         OW_Rom[z] = 0;                      // ROM Array lschen
      }                                      // Ende FOR
   }                                         // Ende wenn Fehler vorhanden
   return Er_Flag;                           // Error-Flag bergeben
}                                            // Ende Read-ROM


// *************************************************************************************
// *   ROUTINE - 1-Wire Skip-ROM (alle Slaves ansprechen)                              *
// *************************************************************************************
void OW_Skip_ROM(uint8_t ow_befehl) {        // Beginn Skip-ROM
   uint8_t ow_tmp = 0;                       // Temp Variable
   // Start mit Reset-Impuls
   ow_tmp = OW_Reset();                      // Reset senden
   if (ow_tmp == 0) {                        // wenn Slave vorhanden, dann...
      // Skip-ROM Befehl senden
      OW_Write(0xCC);                        // Skip-ROM Befehl senden
      // bergebenen Befehl ausfhren
      OW_Write(ow_befehl);                   // Befehl senden
   }                                         // Ende wenn Slave vorhanden
}                                            // Ende Skip-ROM


// *************************************************************************************
// *   ROUTINE - 1-Wire Match-ROM (einen einzelnen Slave ansprechen)                   *
// *************************************************************************************
void OW_Match_ROM(uint8_t ow_befehl) {       // Beginn Match-ROM
   uint8_t ow_tmp = 0;                       // Temp Variable
   uint8_t z = 0;                            // Variable For Schleife
   // Start mit Reset-Impuls
   ow_tmp = OW_Reset();                      // Reset senden
   if (ow_tmp == 0) {                        // wenn Slave vorhanden, dann...
      // Match-ROM Befehl senden
      OW_Write(0x55);                        // Match-ROM Befehl senden
      // ROM Inhalt senden
      for(z=0; z<8; z++) {                   // For z=0 To 7 Step +1
         OW_Write(OW_Rom[z]);                // ROM Inhalt senden
      }                                      // Ende FOR
      // bergebenen Befehl ausfhren
      OW_Write(ow_befehl);                   // Befehl senden
   }                                         // Ende wenn Slave vorhanden
}                                            // Ende Match-ROM


// *************************************************************************************
// *   ROUTINE - 1-Wire Read-Power-Supply (mit Match-ROM)                              *
// *************************************************************************************
unsigned char OW_Power(void) {               // Beginn Read-Power-Supply
   uint8_t ow_byte = 0;                      // eingelesendes Byte
   // Befehl senden
   OW_Match_ROM(0xB4);                       // Befehl "Read Power Supply" senden
   // Daten einlesen
   if (OW_Rd_Bit() > 0) { ow_byte = 1; }     // wenn High, dann 1
   // Rckgabe: 0 = parasitrer Modus
   // Rckgabe: 1 = externe Stromversorgung
   return ow_byte;                           // Byte bergeben
}                                            // Ende Read-Power-Supply


// *************************************************************************************
// *   ROUTINE - 1-Wire Daten in den EEprom schreiben (mit Match-ROM)                  *
// *************************************************************************************
void OW_Write_EEprom(uint8_t ow_Zahl) {      // Beginn EEprom schreiben
   uint8_t z = 0;                            // Variable For Schleife
   // *** erster Schritt ***
   OW_Match_ROM(0x4E);                       // Befehl "Write Scratchpad" senden
   // Daten in Scratchpad schreiben
   for(z=0; z<ow_Zahl; z++) {                // For z=0 To Zahl Step +1
      OW_Write(OW_EEprom[z]);                // EEprom Inhalt senden
   }                                         // Ende FOR
   // Reset Impuls senden (passiert im nchsten Schritt)
   // *** zweiter Schritt ***
   OW_Match_ROM(0x48);                       // Befehl "Copy Scratchpad" senden
   // Zeit fr EEprom Programmierung
   Pause(15);                                // 15ms warten
}                                            // Ende EEprom schreiben












