// *************************************************************************************
// Project:  Tachosignalberwachung mit ATmega88A
// Software: AVR-Studio v4.19 / WinAVR-20100110 / avr-toolchain-installer-3.4.2.1573
// Edit:     Alexander Kilpert / Jens Grtler
// Erstellt: 15.11.2017
// *************************************************************************************


// --- einbinden von Libs --------------------------------------------------------------
#include <avr/io.h>                          // AVR Basisdaten einbinden
#include <avr/interrupt.h>                   // Interrupts bereitstellen
#include <avr/pgmspace.h>                    // Flash zugriff bereitstellen
#include <avr/eeprom.h>                      // EEProm zugriff bereitstellen
#include <avr/wdt.h>                         // Watchdog Routine einbinden

// --- Makros ------------------------------------------------
#define Setzen(adr,bit)   (adr |= (1<<bit))  // Bit setzen
#define Loeschen(adr,bit) (adr &= ~(1<<bit)) // Bit lschen
#define Toggeln(adr,bit)  (adr ^= (1<<bit))  // Bit Toogeln
#define High(adr,bit)     (adr & (1<<bit))   // gesetzes Bit in Adresse
#define Low(adr,bit)    (!(adr & (1<<bit)))  // gesetzes Bit in Adresse

// --- Definiton der Konstanten --------------------------------------------------------
#define R_P_G            128                 // RS232 Ring-Puffer-Gre
#define BF_P_G           64                  // Befehls-String Puffer-Gre
#define T1_vorladen      65176               // Timer1 vorladen (20ms)
#define Sekunden_2       100                 // 2 Sekunden Timer (100 * 20ms = 2 Sek)
#define Ms_800           40                  // 800ms Timer (40 * 20ms = 0,8 Sek)
#define Ein_verz         250                 // 5 Sekunden Einschaltverzgerung I2C-Bus Anzeige (250 * 20ms = 5 Sek)
#define Trennzeichen     ';'                 // Trennzeichen der Messwerte an RS232
#define I2C_Adr_Anzeige  20                  // I2C-Bus Adresse LED-Anzeige



// --- Definiton der Globalen Variabeln ------------------------------------------------
char RS232_Daten[R_P_G];                     // Array fr RS232 Daten
volatile uint8_t Zeiger_schreiben = 0;       // Zeiger - schreiben auf Ringpuffer
volatile uint8_t Zeiger_lesen = 0;           // Zeiger - lesen auf Ringpuffer
volatile uint8_t Len_Puffer = 0;             // Anzahl der Bytes im RS232-Puffer

unsigned short Temperatur[] = { 600, 600, 600, 600, 600, 600, 600, 600 };  // Array fr Temperaturen
unsigned char Tacho_Daten[] = { 0, 0, 0, 0, 0, 0, 0 };  // Array Tachosignale erfassen
unsigned short AD_Werte[] = { 0, 0 };        // Array fr AD-Werte
unsigned short Sp_Werte[] = { 0, 0 };        // Array fr Spannungswerte

unsigned char Bf_String[BF_P_G + 1];         // Array Befehls-String
volatile uint8_t Bf_Zeiger = 0;              // Zeiger fr String

volatile uint8_t Zeit01 = 0;                 // Zeit-Zhler Nr.1 - 2 Sek Verriegelung
volatile uint8_t Zeit02 = 0;                 // Zeit-Zhler Nr.2 - 800ms Verriegelung
volatile uint16_t Zeit03 = 0;                // Zeit-Zhler Nr.3 - Einschaltverzgerung I2C-Bus Ausgabe
volatile uint16_t Zeit04 = 0;                // Zeit-Zhler Nr.4 - TimeOut Hauptalarm Auslsung

volatile uint8_t ACK = 0;                    // ACK I2C-Bus

volatile uint8_t Daten01 = 0;                // I2C-Bus Daten-Byte Nr.1
         // Bit0 = LED Vdd Kanal1 (0=OFF, 1=ON)
         // Bit1 = LED Vdd Kanal1 (1=Blinken)
         // Bit2 = LED K1 Tacho1 (0=OFF, 1=ON)
         // Bit3 = LED K1 Tacho1 (1=Blinken)
         // Bit4 = LED K1 Tacho2 (0=OFF, 1=ON)
         // Bit5 = LED K1 Tacho2 (1=Blinken)
         // Bit6 = LED K1 Tacho3 (0=OFF, 1=ON)
         // Bit7 = LED K1 Tacho3 (1=Blinken)
volatile uint8_t Daten02 = 0;                // I2C-Bus Daten-Byte Nr.2
         // Bit0 = LED Vdd Kanal2 (0=OFF, 1=ON)
         // Bit1 = LED Vdd Kanal2 (1=Blinken)
         // Bit2 = LED K2 Tacho1 (0=OFF, 1=ON)
         // Bit3 = LED K2 Tacho1 (1=Blinken)
         // Bit4 = LED K2 Tacho2 (0=OFF, 1=ON)
         // Bit5 = LED K2 Tacho2 (1=Blinken)
         // Bit6 = LED K2 Tacho3 (0=OFF, 1=ON)
         // Bit7 = LED K2 Tacho3 (1=Blinken)
volatile uint8_t Daten03 = 0;                // I2C-Bus Daten-Byte Nr.3
         // Bit0 = LED DFM (0=OFF, 1=ON)
         // Bit1 = LED DFM (1=Blinken)
         // Bit2 = LED Alarm (0=OFF, 1=ON) (Hauptalarm ON)
         // Bit3 = LED Alarm (1=Blinken) (Voralarm ON)
         // Bit4 = Piezo Voralarm (0=OFF, 1=Intervall)
         // Bit5 = Piezo Hauptalarm (0=OFF, 1=Intervall)
         // Bit6 = Piezo System ON/OFF (0=OFF, 1=Intervall)
         // Bit7 = frei
volatile uint8_t Daten04 = 0;                // I2C-Bus Daten-Byte Taster
         // Bit0 = Blinkfrequenz 0,3 Sek
         // Bit1 = Intervallzeit 1 Sek
         // Bit2 = frei
         // Bit3 = frei
         // Bit4 = Verriegelung Voralarm
         // Bit5 = Verriegelung Hauptalarm
         // Bit6 = Verriegelung Piezo System ON/OFF
         // Bit7 = Taster entprellt (1=bettigt)
volatile uint8_t Error = 0;                  // Fehlerspeicher
         // 1 = Voralarm Kanal1 Tacho1
		 // 2 = Voralarm Kanal1 Tacho2
		 // 3 = Voralarm Kanal1 Tacho3
		 // 4 = Voralarm Kanal2 Tacho1
		 // 5 = Voralarm Kanal2 Tacho2
		 // 6 = Voralarm Kanal2 Tacho3
		 // 7 = Voralarm DFM
		 // 8 = Hauptalarm
volatile uint8_t Flag01 = 0;                 // System Flag
         // Bit0 = Reset Hauptalarm von RS232
         // Bit1 = Voralarm ON von RS232 
         // Bit2 = Hauptalarm ON von RS232 




// --- Definiton Flash Array ---------------------------------
const unsigned char Text0[] PROGMEM = "Reset...";  // Text im Flash
const unsigned char Text1[] PROGMEM = "Temperatur: ";  // Text im Flash
const unsigned char Text2[] PROGMEM = "Tachosignal: ";  // Text im Flash
const unsigned char Text3[] PROGMEM = "AD-Wert: ";  // Text im Flash
const unsigned char Text4[] PROGMEM = "I2C-Bus Adresse: ";  // Text im Flash
const unsigned char Text5[] PROGMEM = "DS18B20 ROM-Adresse: ";  // Text im Flash
const unsigned char Text6[] PROGMEM = "OK";  // Text im Flash
const unsigned char Text7[] PROGMEM = "Start - I2C-Bus wird durchsucht...";  // Text im Flash
const unsigned char Text8[] PROGMEM = "Adresse DS18B20 Nr.";  // Text im Flash
const unsigned char Text9[] PROGMEM = "Tacho-MIN-Wert: ";  // Text im Flash
const unsigned char Text10[] PROGMEM = "PWM-Wert: ";  // Text im Flash
const unsigned char Text11[] PROGMEM = "Fehler: ";  // Text im Flash
const unsigned char Text12[] PROGMEM = "Hauptalarm TimeOut: ";  // Text im Flash
const unsigned char Text13[] PROGMEM = "Spannung: ";  // Text im Flash
const unsigned char Text14[] PROGMEM = "Relais Voralarm: ";  // Text im Flash
const unsigned char Text15[] PROGMEM = "Relais Hauptalarm: ";  // Text im Flash
const unsigned char Text16[] PROGMEM = "ON";  // Text im Flash
const unsigned char Text17[] PROGMEM = "OFF";  // Text im Flash
const unsigned char Text18[] PROGMEM = "   (Summer aus!)";  // Text im Flash


// --- Definiton Flash Array - String --------------------------------------------------
const unsigned char Str0[] PROGMEM = "SUCHE-";  // String im Flash
const unsigned char Str1[] PROGMEM = "I2C";  // String im Flash
const unsigned char Str2[] PROGMEM = "DS18B20";  // String im Flash
const unsigned char Str3[] PROGMEM = "SAVE-";  // String im Flash
const unsigned char Str4[] PROGMEM = "TEMPSENSOR";  // String im Flash
const unsigned char Str5[] PROGMEM = "MIN";  // String im Flash
const unsigned char Str6[] PROGMEM = "PWM";  // String im Flash
const unsigned char Str7[] PROGMEM = "INFO";  // String im Flash
const unsigned char Str8[] PROGMEM = "VORALARM";  // String im Flash
const unsigned char Str9[] PROGMEM = "ON";  // String im Flash
const unsigned char Str10[] PROGMEM = "OFF";  // String im Flash
const unsigned char Str11[] PROGMEM = "HAUPTALARM";  // String im Flash
const unsigned char Str12[] PROGMEM = "RESET";  // String im Flash
const unsigned char Str13[] PROGMEM = "SYSTEM";  // String im Flash





// --- Definiton Union -----------------------------------------------------------------
union Byte_Word {                            // Name der Union 
   unsigned short un_Word;                   // Word Variable
   unsigned char un_Byte[2];                 // Byte Variabeln
};  
union Byte_Word union_var;                   // Aufruf der Union

// --- Definiton E-Eprom Adressen ----------------------------
uint8_t Dummi_EEprom EEMEM = 0;              // Dummi
uint8_t DS18B20_ROM_EEprom[8][8] EEMEM;      // Array im EEProm - DS18B20
uint8_t MIN_Tacho_EEprom[7] EEMEM;           // Array im EEProm - MIN Tachosignal
uint8_t PWM_Wert_EEprom[2] EEMEM;            // Array im EEProm - PWM-Wert
uint8_t TimeOut_Low_EEprom EEMEM;            // Hauptalarm TimeOut Low-Byte im EEProm
uint8_t TimeOut_High_EEprom EEMEM;           // Hauptalarm TimeOut High-Byte im EEProm



// --- einbinden externer Dateien ----------------------------
#include "Pause.h"                           // Pause Routinen einbinden
#include "RS232_r.h"                         // RS232 Routinen einbinden
#include "CRC8.h"                            // CRC8 Routinen einbinden
#include "1-Wire.h"                          // 1-Wire Routinen einbinden
#include "Soft_I2C_Master.h"                 // Software I2C-Bus-Master Routinen einbinden

// --- Definiton der Routinen ----------------------------------------------------------
void Reset(void);                            // Neustart
void Befehl_ausfuehren(void);                // Beginn RS232 Befehl ausfhren



// *************************************************************************************
// *   INTERRUPT Vektor 14 - Timer1 berlauf (alle 20ms)                               *
// *************************************************************************************
ISR(TIMER1_OVF_vect) {                       // Beginn Timer1 berlauf
   static uint8_t Time = 0;                  // interne 1 Sek Verriegelung
   TCNT1 = T1_vorladen;                      // Timer1 neu vorladen
   if (Zeit01 > 0) { Zeit01--; }             // Zeit runter zhlen
   if (Zeit02 > 0) { Zeit02--; }             // Zeit runter zhlen
   if (Zeit03 > 0) { Zeit03--; }             // Zeit runter zhlen
   // nach 1 Sekunde
   Time++;                                   // hoch zhlen
   if (Time >= 50) {                         // nach 1 Sekunde
      Time = 0;                              // Reset
      if (Zeit04 > 1) { Zeit04--; }          // Zeit runter zhlen
   }                                         // Ende nach 1 Sekunde
}                                            // Ende Timer1 berlauf

// *************************************************************************************
// *   INTERRUPT Vektor 19 - USART-RXC komplett                                        *
// *************************************************************************************
ISR(USART_RX_vect) {                         // Beginn USART-RXC komplett
   RS232_Daten[Zeiger_schreiben] = UDR0;     // RS232 Daten an Puffer bergeben
   Zeiger_schreiben++;                       // Zeiger schreiben um 1 erhhen
   if (Zeiger_schreiben == R_P_G) { Zeiger_schreiben = 0; }  // zum Anfang Ringpuffer
   Len_Puffer++;                             // Anzahl der Bytes im Puffer
}                                            // Ende USART-RXC komplett

// *************************************************************************************
// *   ROUTINE - Ring-Puffer einlesen                                                  *
// *************************************************************************************
unsigned char Ringpuffer_lesen(void) {       // Beginn Ringpuffer einlesen
   cli();                                    // Interrupts dealtivieren
   uint8_t Puffer_Daten = RS232_Daten[Zeiger_lesen];  // Byte im Ringpuffer
   Zeiger_lesen++;                           // RS232-Zeiger um 1 erhhen
   if (Zeiger_lesen == R_P_G) { Zeiger_lesen = 0; }  // zum Anfang Ringpuffer
   Len_Puffer--;                             // Anzahl der Bytes im Puffer
   sei();                                    // Interrupts aktivieren
   return Puffer_Daten;                      // eingelesene Daten Return bergeben
}                                            // Ende Ringpuffer einlesen

// *************************************************************************************
// *   FUNKTION - Reset                                                                *
// *************************************************************************************
void Reset(void) {                           // Beginn Reset
   // Endlosschleife starten - Watchdog auslsen
   while(1) { asm volatile("NOP"); }         // Endlosschleife ausfhren - auf Reset warten
}      


// *************************************************************************************
// *   ROUTINE - Tachosignal einlesen                                                  *
// *************************************************************************************
void Tacho_einlesen(void) {                  // Beginn Tachosignal einlesen
   static unsigned char Ver_Pin;             // Verreigelung der einzelnen Ports
   // --- Kanal1 - Tacho1 ---
   if (Low(PIND,PIND4) && Low(Ver_Pin,0)) {  // wenn K1 Tacho1 erkannt, dann...
      Setzen(Ver_Pin,0);                     // Verriegelung aktivieren
      Tacho_Daten[0]++;                      // Tachosignal +1
   }                                         // Ende K1 Tacho1 erkannt
   if High(PIND,PIND4) { Loeschen(Ver_Pin,0); }  // Reset Verriegellung
   // --- Kanal1 - Tacho2 ---
   if (Low(PIND,PIND7) && Low(Ver_Pin,1)) {  // wenn K1 Tacho2 erkannt, dann...
      Setzen(Ver_Pin,1);                     // Verriegelung aktivieren
      Tacho_Daten[1]++;                      // Tachosignal +1
   }                                         // Ende K1 Tacho2 erkannt
   if High(PIND,PIND7) { Loeschen(Ver_Pin,1); }  // Reset Verriegellung
   // --- Kanal1 - Tacho3 ---
   if (Low(PINB,PINB0) && Low(Ver_Pin,2)) {  // wenn K1 Tacho3 erkannt, dann...
      Setzen(Ver_Pin,2);                     // Verriegelung aktivieren
      Tacho_Daten[2]++;                      // Tachosignal +1
   }                                         // Ende K1 Tacho3 erkannt
   if High(PINB,PINB0) { Loeschen(Ver_Pin,2); }  // Reset Verriegellung
   // --- Kanal2 - Tacho1 ---
   if (Low(PINB,PINB1) && Low(Ver_Pin,3)) {  // wenn K2 Tacho1 erkannt, dann...
      Setzen(Ver_Pin,3);                     // Verriegelung aktivieren
      Tacho_Daten[3]++;                      // Tachosignal +1
   }                                         // Ende K2 Tacho1 erkannt
   if High(PINB,PINB1) { Loeschen(Ver_Pin,3); }  // Reset Verriegellung
   // --- Kanal2 - Tacho2 ---
   if (Low(PINB,PINB2) && Low(Ver_Pin,4)) {  // wenn K2 Tacho2 erkannt, dann...
      Setzen(Ver_Pin,4);                     // Verriegelung aktivieren
      Tacho_Daten[4]++;                      // Tachosignal +1
   }                                         // Ende K2 Tacho2 erkannt
   if High(PINB,PINB2) { Loeschen(Ver_Pin,4); }  // Reset Verriegellung
   // --- Kanal2 - Tacho3 ---
   if (Low(PINB,PINB3) && Low(Ver_Pin,5)) {  // wenn K2 Tacho3 erkannt, dann...
      Setzen(Ver_Pin,5);                     // Verriegelung aktivieren
      Tacho_Daten[5]++;                      // Tachosignal +1
   }                                         // Ende K2 Tacho3 erkannt
   if High(PINB,PINB3) { Loeschen(Ver_Pin,5); }  // Reset Verriegellung
   // --- DFM - Tacho ---
   if (Low(PINC,PINC2) && Low(Ver_Pin,6)) {  // wenn DFM Tacho erkannt, dann...
      Setzen(Ver_Pin,6);                     // Verriegelung aktivieren
      Tacho_Daten[6]++;                      // Tachosignal +1
   }                                         // Ende DFM Tacho erkannt
   if High(PINC,PINC2) { Loeschen(Ver_Pin,6); }  // Reset Verriegellung
}                                            // Ende Tachosignal einlesen

// *************************************************************************************
// *   ROUTINE - Tachosignal Messung starten - 2Sek Verriegelung                       *
// *************************************************************************************
void Tacho_Messung_starten(void) {           // Beginn Tachosignal Start
   int a = 0;                                // Temp Variable
   for (a=0;a<=6;a++) {                      // For 0 To 6
      Tacho_Daten[a] = 0;                    // Reset Array Tachosignale
   }                                         // Next
   Zeit01 = Sekunden_2;                      // Zeit01 neu einstellen
}                                            // Ende Tachosignal Start


// *************************************************************************************
// *   ROUTINE - ADC initialisieren                                                    *
// *************************************************************************************
void ADC_Init(void) {                        // Beginn ADC Initialisieren
   uint16_t result;                          // bergabe
   ADMUX = (1<<REFS0);                       // Kondensator am REF-Pin
// ADCSRA = (1<<ADPS2) | (1<<ADPS1);         // Frequenzvorteiler 64 (8MHz/64=125KHz)
   ADCSRA = (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);  // Frequenzvorteiler 128 (16MHz/128=125KHz)
   ADCSRA |= (1<<ADEN);                      // ADC aktivieren
   ADCSRA |= (1<<ADSC);                      // eine ADC-Wandlung 
   while (ADCSRA & (1<<ADSC) ) { asm volatile("NOP"); }  // auf Abschluss der Konvertierung warten
   result = ADCW;                            // bergabe
   result++;                                 // Warnung beseitigen (Wunused-but-set-variable)
}

// *************************************************************************************
// *   ROUTINE - AD-Wert einlesen                                                      *
// *************************************************************************************
uint16_t ADC_Read_1x(uint8_t channel) {      // Beginn AD-Wert einlesen
   ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);  // Kanal waehlen
   ADCSRA |= (1<<ADSC);                      // eine Wandlung "single conversion"
   while (ADCSRA & (1<<ADSC) ) { asm volatile("NOP"); }  // auf Abschluss der Konvertierung warten
   return ADCW;                              // ADC auslesen und zurckgeben
}                                            // Ende AD-Wert einlesen

// *************************************************************************************
// *   ROUTINE - AD-Mittelwertbildung                                                  *
// *************************************************************************************
uint16_t ADC_Read(uint8_t channel) {         // Beginn AD-Mittelwertberechnung
   uint32_t sum = 0;                         // Summe AD-Werte
   for (uint8_t i=0;i<4;++i) {               // 4 Durchlufe
      sum += ADC_Read_1x(channel);           // AD-Wert aufsummieren
   }                                         // Ende For
   return (uint16_t)(sum / 4);               // bergabe AD-Mittelwert
}                                            // Ende AD-Mittelwertberechnung

// *************************************************************************************
// *   ROUTINE - AD-Wert in Spannung umwandeln                                         *
// *************************************************************************************
uint8_t Spannung_Out(uint16_t AD_w) {        // Beginn Spannung-Out
   int32_t Ergebnis = 0;                     // Ergebnis
   int16_t Tmp = 0;                          // Temp Variable
   Ergebnis = (int32_t)AD_w * 147;           // AD-Wert * Spannungsteiler (& 10000)
   Tmp = Ergebnis % 1000;                    // Kommastelle erfassen
   Ergebnis /= 1000;                         // Kommastelle abziehen
   if (Tmp >= 500) { Ergebnis++; }           // wenn grer 0,5, dann +1
   if (Ergebnis > 255) { Ergebnis = 255; }   // obere Grenze festlegen
   if (Ergebnis < 0) { Ergebnis = 0; }       // untere Grenze festlegen
   return (uint8_t)Ergebnis;                 // Spannung zurckgeben
}                                            // Ende Spannung-Out

// *************************************************************************************
// *   ROUTINE - AD-Werte einlesen                                                     *
// *************************************************************************************
void AD_Wert_einlesen(void) {                // Beginn AD-Werte einlesen
   // Kanal1
   if (OCR0A > 0) {                          // wenn PWM-Kanal1 EIN, dann...
      AD_Werte[0] = ADC_Read(0);             // AD-Wert bergeben  
      Sp_Werte[0] = Spannung_Out(AD_Werte[0]);  // AD-Wert in Spannung umrechnen  
   }                                         // Ende PWM-Kanal1 EIN   
   else { AD_Werte[0] = 0; }                 // wenn PWM-Kanal1 AUS, dann Null
   // Kanal2
   if (OCR0B > 0) {                          // wenn PWM-Kanal2 EIN, dann... 
      AD_Werte[1] = ADC_Read(1);             // AD-Wert bergeben 
      Sp_Werte[1] = Spannung_Out(AD_Werte[1]);  // AD-Wert in Spannung umrechnen
   }                                         // Ende PWM-Kanal2 EIN   
   else { AD_Werte[1] = 0; }                 // wenn PWM-Kanal2 AUS, dann Null
}                                            // Ende AD-Werte einlesen







// *************************************************************************************
// *   ROUTINE - 1-Wire mehrere Bytes lesen und in Array speichern                     *
// *************************************************************************************
void OW_Read_Data(uint8_t ow_Zahl) {         // Beginn Read-Data
   uint8_t z = 0;                            // Variable For Schleife
   // mehrere Bytes einlesen und speichern
   for(z=0; z<ow_Zahl; z++) {                // For z=0 To Zahl Step +1
      OW_Data[z] = OW_Read();                // Byte einlesen
   }                                         // Ende FOR
}                                            // Ende Read-Data

// *************************************************************************************
// *   ROUTINE - 1-Wire ROM Adresse aus EEprom einlesen                                *
// *************************************************************************************
void ROM_aus_EEprom_laden(uint8_t woher) {   // Beginn ROM-EEprom lesen
   uint8_t z = 0;                            // Variable For Schleife
   for(z=0; z<8; z++) {                      // For z=0 To 7 Step +1
      OW_Rom[z] = eeprom_read_byte(&DS18B20_ROM_EEprom[woher][z]);  // Byte aus EEprom einlesen
   }                                            // Ende FOR
}                                            // Ende ROM-EEprom lesen

// *************************************************************************************
// *   ROUTINE - 1-Wire Temperatur berechnen                                           *
// *************************************************************************************
void OW_Temperatur(uint8_t wohin) {          // Beginn Temperatur berechnen
   static unsigned char Temp_Fsp[8];         // Fehlerspeicher falsche Temperatur
   uint8_t Grad = 0;                         // Speicher Grad
   uint16_t Komma = 0;                       // Speicher Kommastelle
   uint16_t Mod = 0;                         // Temp Variable
   uint16_t T1 = 0;                          // Temp Variable
   // *** Family-Code DS18B20 (0x28) ***
   if (OW_Rom[0] == 0x28) {                  // wenn DS18B20, dann...
      // 16-Bit Variable laden
      T1 = OW_Data[0];                       // Low-Byte
      T1 |= OW_Data[1] << 8;                 // High-Byte
      // Minusgrade erfassen
      if (OW_Data[1] & 0x08) {               // wenn Minusgrade, dann...
         T1 = ~T1 + 1;                       // invertieren & +1
      }                                      // Ende wenn Minusgrade
      // Grandzahl berechnen
      Grad = (T1 & 0x07FF) >> 4;             // 12-Bit Maske & Kommastelle entfernen
      // auf 4 Kommastellen berechnen
      Komma = T1 & 0x000F;                   // unteren 4-Bits bergeben
      Komma *= 625;                          // 12-Bit = 0,625
      // auf 1 Kommastelle runden
      Mod = Komma % 1000;                    // MOD 1000
      Komma /= 1000;                         // durch 1000 teilen
      if (Mod >= 500 ) { Komma++; }          // Wenn MOD > 5 dann aufrunden
   }                                         // Ende wenn DS18B20
   // *** CRC8 berechnen ***
   CRC8_Sp = 0;                              // CRC8 Speicher lschen
   for(T1=0; T1<8; T1++) {                   // For z=0 To 8 Step +1
      CRC8_berechnen(OW_Data[T1]);           // CRC8 berechnen
   }                                         // Ende FOR
   if (CRC8_Sp != OW_Data[8]) {              // wenn CRC nicht stimmt, dann...
      if (Temp_Fsp[wohin] < 255) { Temp_Fsp[wohin] += 1; }  // Fehlerspeicher hoch zhlen
      if (Temp_Fsp[wohin] >= 5) {            // nach 5x falschen Werten...
         Temperatur[wohin] = 1270;           // Fehler speichern (127,0C)
      }                                      // Ende nach 3x falschen Werten
   }                                         // Ende wenn CRC nicht stimmt
   // *** Tempertur in Array schreiben ***
   else {                                    // wenn CRC OK, dann...
      Temperatur[wohin] = (Grad * 10) + Komma;  // Temperatur speichern
      Temp_Fsp[wohin] = 0;                   // Reset Fehlerspeicher
   }                                         // Ende wenn CRC OK
}                                            // Ende Temperatur berechnen

// *************************************************************************************
// *   ROUTINE - 1-Wire Temperaturmessung starten (Dauer 800ms)                        *
// *************************************************************************************
void Temperatur_starten(void) {              // Beginn Temperaturmessung starten
   OW_Skip_ROM(0x44);                        // DS18S20 Temperaturmessung starten
   Zeit02 = Ms_800;                          // Zeit02 neu einstellen
}                                            // Ende Temperaturmessung starten

// *************************************************************************************
// *   ROUTINE - 1-Wire Temperatur einlesen                                            *
// *************************************************************************************
void Temperatur_einlesen(void) {             // Beginn Temperatur einlesen
   int a = 0;                                // Temp Variable
   // warten bis Temperaturmessung fertig ist
   while (Zeit02) { asm volatile("NOP"); }   // auf Abschluss der Temperaturmessung warten
   // Temperaturen einlesen
   for(a=0; a<8; a++) {                      // For 0 To 7 Step +1
      ROM_aus_EEprom_laden(a);               // ROM Daten aus EEprom einlesen
      OW_Match_ROM(0xBE);                    // Scratch-Pad auslesen
      OW_Read_Data(9);                       // 9-Bytes einlesen
      OW_Temperatur(a);                      // Temperatur berechen & anzeigen
   }                                         // Next
}                                            // Ende Temperatur einlesen



// *************************************************************************************
// *   ROUTINE - RS232 Daten anzeigen                                                  *
// *************************************************************************************
void RS232_Daten_ausgeben(void) {            // Beginn RS232 Anzeige
   int a = 0;                                // Temp Variable

   // Start - Einschaltverzgerung
   if (Zeit03 > 0) { return; }               // nach Systemstart 5 Sek keine Daten senden

   // Temperatur Daten
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   RS232_Array(&Text1[0]);                   // Temperatur - Ausgabe an RS232
   for(a=0; a<8; a++) {                      // For 0 To 7 Step +1
      RS232_Zahl(Temperatur[a]);             // Grad - Ausgabe an RS232
      if (a<7) { RS232_Put(Trennzeichen); }  // Trennzeichen - Ausgabe an RS232
   }                                         // Next
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // Tachosignale
   RS232_Array(&Text2[0]);                   // Tachosignal - Ausgabe an RS232
   for(a=0; a<7; a++) {                      // For 0 To 6 Step +1
      RS232_Zahl(Tacho_Daten[a]);            // Impulse - Ausgabe an RS232
      if (a<6) { RS232_Put(Trennzeichen); }  // Trennzeichen - Ausgabe an RS232
   }                                         // Next
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // PWM-Wert
   RS232_Array(&Text10[0]);                  // PWM-Werte - Ausgabe an RS232
   RS232_Zahl(OCR0A);                        // Wert1 - Ausgabe an RS232
   RS232_Put(Trennzeichen);                  // Trennzeichen - Ausgabe an RS232
   RS232_Zahl(OCR0B);                        // Wert2 - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // AD-Werte
   RS232_Array(&Text3[0]);                   // AD-Werte - Ausgabe an RS232
   RS232_Zahl(AD_Werte[0]);                  // Wert1 - Ausgabe an RS232
   RS232_Put(Trennzeichen);                  // Trennzeichen - Ausgabe an RS232
   RS232_Zahl(AD_Werte[1]);                  // Wert2 - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // Spannung
   RS232_Array(&Text13[0]);                  // Spannung - Ausgabe an RS232
   RS232_Zahl(Sp_Werte[0]);                  // Wert1 - Ausgabe an RS232
   RS232_Put(Trennzeichen);                  // Trennzeichen - Ausgabe an RS232
   RS232_Zahl(Sp_Werte[1]);                  // Wert2 - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // Error
   RS232_Array(&Text11[0]);                  // Error - Ausgabe an RS232
   RS232_Zahl(Error);                        // Error - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // Voralarm
   RS232_Array(&Text14[0]);                  // Relais Voralarm - Ausgabe an RS232
   if High(PORTD,PD2) { RS232_Array(&Text16[0]); }  // ON
   else { RS232_Array(&Text17[0]); }         // OFF
   if High(Daten04,4) { RS232_Array(&Text18[0]); }  // Summer aus! - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // Hauptalarm
   RS232_Array(&Text15[0]);                  // Relais Hauptalarm - Ausgabe an RS232
   if High(PORTD,PD3) { RS232_Array(&Text16[0]); }  // ON
   else { RS232_Array(&Text17[0]); }         // OFF
   if High(Daten04,5) { RS232_Array(&Text18[0]); }  // Summer aus! - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch

}                                            // Ende RS232 Anzeige








// *************************************************************************************
// *   ROUTINE - Voralarm ON                                                           *
// *************************************************************************************
void Voralarm_ON(void) {                     // Beginn Voralarm ON
   Setzen(Daten03,2);                        // LED Alarm ON
   if Low(Daten03,5) { Setzen(Daten03,3); }  // wenn Hauptalarm aus, dann LED Alarm blinken ON
   Setzen(Daten03,4);                        // Piezo Voralarm ON
   Setzen(PORTD,PD2);                        // Relais Voralarm ON
   // TimeOut Hauptalarm starten
   if ((Zeit04 == 0) && Low(Flag01,1)) {     // Erststart (und Hand Voralarm aus)
      union_var.un_Byte[1] = eeprom_read_byte(&TimeOut_High_EEprom);  // High-Byte Hauptalarm TimeOut aus EEprom einlesen
      union_var.un_Byte[0] = eeprom_read_byte(&TimeOut_Low_EEprom);   // Low-Byte Hauptalarm TimeOut aus EEprom einlesen
      Zeit04 = union_var.un_Word;            // TimeOut bergeben
   }                                         // Ende Erststart
}                                            // Ende Voralarm ON

// *************************************************************************************
// *   ROUTINE - Voralarm OFF                                                          *
// *************************************************************************************
void Voralarm_OFF(void) {                    // Beginn Voralarm OFF
   if Low(Daten03,5) {                       // wenn Hauptalarm aus, dann...
      Loeschen(Daten03,2);                   // LED Alarm OFF
      Loeschen(Daten03,3);                   // LED Alarm blinken OFF
   }                                         // Ende Hauptalarm aus
   Loeschen(Daten03,4);                      // Piezo Voralarm OFF
   Loeschen(PORTD,PD2);                      // Relais Voralarm OFF
   Zeit04 = 0;                               // TimeOut Hauptalarm deaktivieren
}                                            // Ende Voralarm OFF

// *************************************************************************************
// *   ROUTINE - Hauptalarm ON                                                         *
// *************************************************************************************
void Hauptalarm_ON(void) {                   // Beginn Hauptalarm ON
   Setzen(Daten03,2);                        // LED Alarm ON
   Loeschen(Daten03,3);                      // LED Alarm blinken OFF
   Setzen(Daten03,5);                        // Piezo Hauptalarm ON
   Setzen(PORTD,PD3);                        // Relais Hauptalarm ON
}                                            // Ende Hauptalarm ON

// *************************************************************************************
// *   ROUTINE - Hauptalarm OFF                                                        *
// *************************************************************************************
void Hauptalarm_OFF(void) {                  // Beginn Hauptalarm OFF
   if Low(Daten03,4) {                       // wenn Voralarm aus, dann...
      Loeschen(Daten03,2);                   // LED Alarm OFF
      Loeschen(Daten03,3);                   // LED Alarm blinken OFF
   }                                         // Ende Voralarm aus
   Loeschen(Daten03,5);                      // Piezo Hauptalarm OFF
   Loeschen(PORTD,PD3);                      // Relais Haupralarm OFF
   Zeit04 = 0;                               // TimeOut Hauptalarm deaktivieren
}                                            // Ende Hauptalarm OFF

// *************************************************************************************
// *   ROUTINE - Systemalarm ON                                                        *
// *************************************************************************************
void Systemalarm_ON(void) {                  // Beginn Systemalarm ON
   Setzen(Daten03,2);                        // LED Alarm ON
   Setzen(Daten03,3);                        // LED Alarm blinken ON
   Setzen(Daten03,6);                        // Piezo Systemalarm ON
}                                            // Ende Systemalarm ON

// *************************************************************************************
// *   ROUTINE - Systemalarm OFF                                                       *
// *************************************************************************************
void Systemalarm_OFF(void) {                 // Beginn Systemalarm OFF
   Loeschen(Daten03,2);                      // LED Alarm OFF
   Loeschen(Daten03,3);                      // LED Alarm blinken OFF
   Loeschen(Daten03,6);                      // Piezo Systemalarm OFF
}                                            // Ende Systemalarm OFF








// *************************************************************************************
// *   ROUTINE - I2C Daten verarbeiten und an LED-Anzeige senden                       *
// *************************************************************************************
void I2C_Daten_verarbeiten_senden(void) {    // Beginn I2C Daten senden
   static uint8_t  ZZT = 0;                  // Zeit-Zhler-Taster von LED-Anzeige
   uint8_t  MIN = 0;                         // MIN-Wert Tachosignale

   // Start - Einschaltverzgerung
   if (Zeit03 > 0) { return; }               // nach Systemstart 5 Sek keine Daten senden

   // Voreinstellung
   Error = 0;                                // Reset

   // Taster-Daten einlesen
   ACK = Soft_I2C_Start(I2C_Adr_Anzeige + 1);  // I2C-Start und Lese-Adresse senden
   if (ACK == 1) {                           // wenn Adresse OK, dann...
      Daten04 = Soft_I2C_Read(0);            // Taster Flag einlesen, NACK senden
   }                                         // Ende Adresse OK
   Soft_I2C_Stop();                          // I2C-Stop

   // Kanal1 - Vdd einstellen
   if (OCR0A > 0) {                          // wenn PWM-Kanal1 EIN, dann...
      Setzen(Daten01,0);                     // LED Vdd ON

      // Kanal1 - Tacho1 - MIN-Wert verarbeiten
      MIN = eeprom_read_byte(&MIN_Tacho_EEprom[0]);  // MIN-Wert Tacho1 aus EEprom einlesen
      if (MIN > 0) {                         // wenn Tachosignal vorhanden, dann...
         Setzen(Daten01,2);                  // LED Tacho1 ON
         if (Tacho_Daten[0] >= MIN) {        // wenn Tachosignal >= MIN-Wert, dann...
            Loeschen(Daten01,3);             // LED Tacho1 blinken OFF
         }                                   // Ende Tachosignal >= MIN-Wert
         else {                              // wenn Tachosignal Error, dann...
            Setzen(Daten01,3);               // LED Tacho1 blinken ON
            Error = 1;                       // Error Tacho1
         }                                   // Ende Tachosignal Error
      }                                      // Ende Tachosignal vorhanden
      else {                                 // wenn kein Tachosignal, dann...
         Loeschen(Daten01,2);                // LED Tacho1 OFF
         Loeschen(Daten01,3);                // LED Tacho1 blinken OFF
      }                                      // Ende kein Tachosignal

      // Kanal1 - Tacho2 - MIN-Wert verarbeiten
      MIN = eeprom_read_byte(&MIN_Tacho_EEprom[1]);  // MIN-Wert Tacho2 aus EEprom einlesen
      if (MIN > 0) {                         // wenn Tachosignal vorhanden, dann...
         Setzen(Daten01,4);                  // LED Tacho2 ON
         if (Tacho_Daten[1] >= MIN) {        // wenn Tachosignal >= MIN-Wert, dann...
            Loeschen(Daten01,5);             // LED Tacho2 blinken OFF
         }                                   // Ende Tachosignal >= MIN-Wert
         else {                              // wenn Tachosignal Error, dann...
            Setzen(Daten01,5);               // LED Tacho2 blinken ON
            Error = 2;                       // Error Tacho2
         }                                   // Ende Tachosignal Error
      }                                      // Ende Tachosignal vorhanden
      else {                                 // wenn kein Tachosignal, dann...
         Loeschen(Daten01,4);                // LED Tacho2 OFF
         Loeschen(Daten01,5);                // LED Tacho2 blinken OFF
      }                                      // Ende kein Tachosignal

      // Kanal1 - Tacho3 - MIN-Wert verarbeiten
      MIN = eeprom_read_byte(&MIN_Tacho_EEprom[2]);  // MIN-Wert Tacho3 aus EEprom einlesen
      if (MIN > 0) {                         // wenn Tachosignal vorhanden, dann...
         Setzen(Daten01,6);                  // LED Tacho2 ON
         if (Tacho_Daten[2] >= MIN) {        // wenn Tachosignal >= MIN-Wert, dann...
            Loeschen(Daten01,7);             // LED Tacho3 blinken OFF
         }                                   // Ende Tachosignal >= MIN-Wert
         else {                              // wenn Tachosignal Error, dann...
            Setzen(Daten01,7);               // LED Tacho3 blinken ON
            Error = 3;                       // Error Tacho3
         }                                   // Ende Tachosignal Error
      }                                      // Ende Tachosignal vorhanden
      else {                                 // wenn kein Tachosignal, dann...
         Loeschen(Daten01,6);                // LED Tacho3 OFF
         Loeschen(Daten01,7);                // LED Tacho3 blinken OFF
      }                                      // Ende kein Tachosignal

   }                                         // Ende PWM-Kanal1 EIN
   else {                                    // wenn PWM-Kanal1 AUS, dann...
      Daten01 = 0;                           // alle LEDs OFF
   }                                         // Ende PWM-Kanal1 AUS

   // Kanal2 - Vdd einstellen
   if (OCR0B > 0) {                          // wenn PWM-Kanal2 EIN, dann...
      Setzen(Daten02,0);                     // LED Vdd ON

      // Kanal2 - Tacho1 - MIN-Wert verarbeiten
      MIN = eeprom_read_byte(&MIN_Tacho_EEprom[3]);  // MIN-Wert Tacho1 aus EEprom einlesen
      if (MIN > 0) {                         // wenn Tachosignal vorhanden, dann...
         Setzen(Daten02,2);                  // LED Tacho1 ON
         if (Tacho_Daten[3] >= MIN) {        // wenn Tachosignal >= MIN-Wert, dann...
            Loeschen(Daten02,3);             // LED Tacho1 blinken OFF
         }                                   // Ende Tachosignal >= MIN-Wert
         else {                              // wenn Tachosignal Error, dann...
            Setzen(Daten02,3);               // LED Tacho1 blinken ON
            Error = 4;                       // Error Tacho1
         }                                   // Ende Tachosignal Error
      }                                      // Ende Tachosignal vorhanden
      else {                                 // wenn kein Tachosignal, dann...
         Loeschen(Daten02,2);                // LED Tacho1 OFF
         Loeschen(Daten02,3);                // LED Tacho1 blinken OFF
      }                                      // Ende kein Tachosignal

      // Kanal2 - Tacho2 - MIN-Wert verarbeiten
      MIN = eeprom_read_byte(&MIN_Tacho_EEprom[4]);  // MIN-Wert Tacho2 aus EEprom einlesen
      if (MIN > 0) {                         // wenn Tachosignal vorhanden, dann...
         Setzen(Daten02,4);                  // LED Tacho2 ON
         if (Tacho_Daten[4] >= MIN) {        // wenn Tachosignal >= MIN-Wert, dann...
            Loeschen(Daten02,5);             // LED Tacho2 blinken OFF
         }                                   // Ende Tachosignal >= MIN-Wert
         else {                              // wenn Tachosignal Error, dann...
            Setzen(Daten02,5);               // LED Tacho2 blinken ON
            Error = 5;                       // Error Tacho3
         }                                   // Ende Tachosignal Error
      }                                      // Ende Tachosignal vorhanden
      else {                                 // wenn kein Tachosignal, dann...
         Loeschen(Daten02,4);                // LED Tacho2 OFF
         Loeschen(Daten02,5);                // LED Tacho2 blinken OFF
      }                                      // Ende kein Tachosignal

      // Kanal2 - Tacho3 - MIN-Wert verarbeiten
      MIN = eeprom_read_byte(&MIN_Tacho_EEprom[5]);  // MIN-Wert Tacho3 aus EEprom einlesen
      if (MIN > 0) {                         // wenn Tachosignal vorhanden, dann...
         Setzen(Daten02,6);                  // LED Tacho3 ON
         if (Tacho_Daten[5] >= MIN) {        // wenn Tachosignal >= MIN-Wert, dann...
            Loeschen(Daten02,7);             // LED Tacho3 blinken OFF
         }                                   // Ende Tachosignal >= MIN-Wert
         else {                              // wenn Tachosignal Error, dann...
            Setzen(Daten02,7);               // LED Tacho3 blinken ON
            Error = 6;                       // Error Tacho3
         }                                   // Ende Tachosignal Error
      }                                      // Ende Tachosignal vorhanden
      else {                                 // wenn kein Tachosignal, dann...
         Loeschen(Daten02,6);                // LED Tacho3 OFF
         Loeschen(Daten02,7);                // LED Tacho3 blinken OFF
      }                                      // Ende kein Tachosignal

   }                                         // Ende PWM-Kanal2 EIN
   else {                                    // wenn PWM-Kanal2 AUS, dann...
      Daten02 = 0;                           // alle LEDs OFF
   }                                         // Ende PWM-Kanal2 AUS

   // DFM - MIN-Wert verarbeiten
   MIN = eeprom_read_byte(&MIN_Tacho_EEprom[6]);  // MIN-Wert DFM aus EEprom einlesen
   if (MIN > 0) {                            // wenn Tachosignal vorhanden, dann...
      Setzen(Daten03,0);                     // LED DFM ON
      if (Tacho_Daten[6] >= MIN) {           // wenn Tachosignal >= MIN-Wert, dann...
         Loeschen(Daten03,1);                // LED DFM blinken OFF
      }                                      // Ende Tachosignal >= MIN-Wert
      else {                                 // wenn Tachosignal Error, dann...
         Setzen(Daten03,1);                  // LED DFM blinken ON
         Error = 7;                          // Error DFM
      }                                      // Ende Tachosignal Error
   }                                         // Ende Tachosignal vorhanden
   else {                                    // wenn kein Tachosignal, dann...
      Loeschen(Daten03,0);                   // LED DFM OFF
      Loeschen(Daten03,1);                   // LED DFM blinken OFF
   }                                         // Ende kein Tachosignal

   // TimeOut Hauptalarm verarbeiten
   if High(Daten04,7) { ZZT++; }             // wenn Taster dauerhaft bettigt, dann ZZT + 1
   else { ZZT = 0; }                         // Reset
   if Low(Flag01,2) {                        // wenn Hand Hauptalarm OFF, dann...
      if (High(Flag01,0) || (ZZT >= 3)) {    // wenn Hauptalarm quitiert, dann...
         Hauptalarm_OFF();                   // Hauptalarm OFF
         Loeschen(Flag01,0);                 // Reset Flag
      }                                      // Ende Hauptalarm quitiert
      if (Zeit04 == 1) {                     // wenn Haupralarm TimeOut erreicht
         Hauptalarm_ON();                    // Hauptalarm ON
         Error = 8;                          // Error Hauptalarm
      }                                      // Ende Haupralarm TimeOut erreicht
   }                                         // Ende wenn Hand Hauptalarm OFF
   else {                                    // wenn Hand Hauptalarm ON, dann...
      if (ZZT >= 3) {                        // wenn Hauptalarm quitiert, dann...
         Hauptalarm_OFF();                   // Hauptalarm OFF
         Loeschen(Flag01,2);                 // Reset - Hand Hauptalarm deaktivieren
      }                                      // Ende Hauptalarm quitiert
      else { Hauptalarm_ON(); }              // Hauptalarm ON
   }                                         // Ende wenn Hand Hauptalarm ON

   // Voralarm verarbeiten
   if (Error == 0) {                         // wenn OK, dann...
      if High(Flag01,1) {                    // wenn Hand Voralarm ON, dann...
         Voralarm_ON();                         // Voralarm ON
      }                                      // Ende Hand Voralarm ON
      else { Voralarm_OFF(); }               // Voralarm OFF
   }                                         // Ende wenn OK
   if ((Error >= 1) && (Error <= 7)) {       // wenn Error Voralarm, dann...
      Loeschen(Flag01,1);                    // Reset - Hand Voralarm deaktivieren
      Loeschen(Flag01,2);                    // Reset - Hand Hauptalarm deaktivieren
      Voralarm_ON();                         // Voralarm ON
   }                                         // Ende Error Voralarm 

   // Daten senden
   ACK = Soft_I2C_Start(I2C_Adr_Anzeige);    // I2C-Start und Schreib-Adresse senden
   if (ACK == 1) {                           // wenn Adresse OK, dann...
      Soft_I2C_Write(Daten01);               // Daten Kanal2 senden
      Soft_I2C_Write(Daten02);               // Daten Kanal2 senden
      Soft_I2C_Write(Daten03);               // Daten DFM & Alarm senden
   }                                         // Ende Adresse OK
   Soft_I2C_Stop();                          // I2C-Stop

}                                            // Ende I2C Daten senden







// *************************************************************************************
// *   ROUTINE - Reset Befehlsstring                                                   *
// *************************************************************************************
void Reset_Befehl(void) {                    // Beginn Reset Befehlsstring
   int a;                                    // For Variable
   for(a=0; a<=BF_P_G; a++) {                // For a=0 To Ende Befehl Step +1
      Bf_String[a] = 0;                      // Reset - Null in String schreiben
   }                                         // Ende FOR
   Bf_Zeiger = 0;                            // Reset Zeiger
}                                            // Ende Reset Befehlsstring

// *************************************************************************************
// *   ROUTINE - RS232 Daten vom Ringpuffer abholen                                    *
// *************************************************************************************
void RS232_einlesen(void) {                  // Beginn RS232 Daten einlesen
   uint8_t  Data = 0;                        // Daten
   int Wh = 1;                               // While Schleife
   // Befehls-String speichern
   while(Wh) {                               // ausfhren bis 0
      Data = Ringpuffer_lesen();             // Byte vom Ringpuffer abholen
      if ((Data >= 32) && (Data <= 126)) {   // wenn sichtbare Zeichen, dann...
         Bf_String[Bf_Zeiger] = Data;        // Daten in String schreiben
         if (Bf_Zeiger < BF_P_G) { Bf_Zeiger++; }  // String Zeiger +1
      }                                      // Ende wenn sichtbare Zeichen
      // Ende
      if ((Len_Puffer == 0) || (Data == 13)) { Wh = 0; }  // While verlassen
   }                                         // Ende While
   // Befehl verarbeiten
   if (Data == 13) { Befehl_ausfuehren(); }  // Befehl ausfhren
}                                            // Ende RS232 Daten einlesen










// *************************************************************************************
// *   ROUTINE - Zahl aus String erstellen                                             *
// *************************************************************************************
uint16_t Zahl_erstellen(uint8_t Z_Zeiger) {  // Beginn Zahl erstellen
   uint16_t Z_Ergebnis = 0;                  // fertige Zahl
   uint8_t yy = 1;                           // While Variable
   while (yy) {                              // ausfhren bis y = 0
      if ((Bf_String[Z_Zeiger] >= '0') && (Bf_String[Z_Zeiger] <= '9')) {  // wenn Zahl, dann...
         Z_Ergebnis = (Z_Ergebnis * 10) + (Bf_String[Z_Zeiger] - 48);  // Zahl erstellen
         Z_Zeiger++;                         // nchste Array einstellen
      }                                      // Ende wenn Zahl
      else { yy = 0; }                       // wenn keine Zahl, dann While verlassen
   }                                         // Ende while
   return Z_Ergebnis;                        // eingelesene Daten Return bergeben
}                                            // Ende Zahl erstellen

// *************************************************************************************
// *   ROUTINE - Zeiger auf die nchste Zahl einstellen                                *
// *************************************************************************************
uint8_t naechste_Zahl(uint8_t N_Zahl) {      // Beginn nchste Zahl
   uint8_t yy = 1;                           // While Variable
   uint8_t yy_Flag = 0;                      // Flag Verriegelung
   while (yy) {                              // ausfhren bis y = 0
      if ((Bf_String[N_Zahl] >= '0') && (Bf_String[N_Zahl] <= '9')) {  // wenn Zahl, dann...
         if (yy_Flag == 0) {                 // wenn Zahl, dann... 
            N_Zahl++;                        // Zeiger weiter hoch zhlen
         }                                   // Ende wenn Zahl
         else { yy = 0;}                     // While verlassen
      }                                      // Ende wenn Zahl
      else {                                 // wenn keine Zahl, dann...
	     N_Zahl++;                           // Zeiger erhhen
         yy_Flag = 1;                        // zur nchsten Zahl springen
      }                                      // Ende wenn keine Zahl
      if (N_Zahl > BF_P_G) {                 // Sicherheit - Array vor berlauf schtzen
	     N_Zahl = BF_P_G;                    // Sicherheit - zurck setzen
         yy = 0;                             // Sicherheit - While verlassen
	  }                                      // Sicherheit - Array vor berlauf schtzen
   }                                         // Ende while
   return N_Zahl;                            // eingelesene Daten Return bergeben
}                                            // Ende nchste Zahl

// *************************************************************************************
// *   ROUTINE - String vergleichen                                                    *
// *************************************************************************************
unsigned char Str_vergleich(uint8_t BfZig, const unsigned char *Zig) {  // Beginn String vergleichen
   uint8_t Data = 0;                         // Daten
   uint8_t Ergebnis = 1;                     // 1=bereinstimmung
   uint8_t Gr_Str = 0;                       // Buchstaben Gro vom Befehls-String
   do {                                      // ausfhren bis Daten = 0
      Data = pgm_read_byte(Zig);             // Byte aus Flash einlesen
      if (Data != 0) {                       // wenn Daten vorhanden
         Gr_Str = Bf_String[BfZig];          // Byte aus Befehls-String einlesen
         if ((Gr_Str >= 97) && (Gr_Str <= 122)) { // wenn Kleinbuchstaben, dann...
            Gr_Str -= 32;                    // in Grobuchstabe umwandeln
         }                                   // Ende wenn Kleinbuchstaben
         if (Gr_Str != Data) {               // wenn Daten nicht bereinstimmen, dann...
            Ergebnis = 0;                    // keine bereinstimmung
            Data = 0;                        // While verlassen
         }                                   // Ende wenn Daten nicht bereinstimmen
      }                                      // Ende wenn Daten vorhanden
      Zig++;                                 // Zeiger auf nchstes Byte
      BfZig++;                               // Zeiger auf nchstes Byte
   } while (Data);                           // End While
   return Ergebnis;                          // bergabe Ergebnis (0=falsch & 1=wahr)
}                                            // Ende String vergleichen

// *************************************************************************************
// *   ROUTINE - String in Grobuchstaben umwandeln                                    *
// *************************************************************************************
void Str_Upper(void) {                       // Beginn String in Grobuchstaben
   uint8_t  Data = 0;                        // Daten
   int Wh = BF_P_G + 1;                      // Voreinstellung While
   while (Wh) {                              // ausfhren bis 0
      Wh--;                                  // weiter im String
      Data = Bf_String[Wh];                  // Befehlsstring einlesen
      if ((Data >= 97) && (Data <= 122)) {   // wenn Kleinbuchstaben, dann...
         Bf_String[Wh] = Data - 32;          // Grobuchstabe in String schreiben
      }                                      // Ende wenn Kleinbuchstaben
   }                                         // Ende While
}                                            // Ende String in Grobuchstaben

// *************************************************************************************
// *   ROUTINE - RS232 Befehl ausfhren                                                *
// *************************************************************************************
void Befehl_ausfuehren(void) {               // Beginn RS232 Befehl ausfhren
   uint8_t OK_Flag = 0;                      // OK Flag
   uint8_t Tmp_a = 1;                        // Temp Variable - Voreinstellung fr While
   uint8_t Tmp_b = 0;                        // Temp Variable - Voreinstellung fr While
   uint16_t Tmp_c = 0;                       // Temp Variable

   // String berprfen
   if (Bf_String[1] == 0) { return; }        // zurck wenn nicht mindestens zwei Zeichen im String
   Str_Upper();                              // Strimg in Grobuchstaben umwandeln

   // --- SUCHE- -----------------------------------------------------------------------
   if (Str_vergleich(0,&Str0[0])) {          // wenn SUCHE- erkannt wurde
      // SUCHE-I2C
      if (Str_vergleich(6,&Str1[0])) {      // wenn I2C erkannt wurde
         RS232_Array(&Text7[0]);            // Start - I2C-Bus wird durchsucht - Ausgabe an RS232
         RS232_Zeilenumbruch();             // Zeilenumbruch
         for (Tmp_c=2;Tmp_c<=254;Tmp_c+=2) { // For 2 To 254 Stepp 2
            wdt_reset();                     // Watchdog Reset
            ACK = Soft_I2C_Start(Tmp_c);     // I2C-Start und Schreib-Adresse senden
            if (ACK > 0) {                   // wenn Baustein gefunden, dann...
               RS232_Array(&Text4[0]);       // I2C-Bus Adresse - Ausgabe an RS232
               RS232_Zahl(Tmp_c);            // Adresse - Ausgabe an RS232
               RS232_Zeilenumbruch();        // Zeilenumbruch
            }                                // Ende wenn Baustein gefunden
            Soft_I2C_Stop();                 // I2C-Stop
         }                                   // Ende For
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende I2C erkannt wurde
      // SUCHE-DS18B20
      else if (Str_vergleich(6,&Str2[0])) {  // wenn DS18B20 erkannt wurde
         OW_Read_ROM();                      // ROM-Adresse einlesen
         RS232_Array(&Text5[0]);             // DS18B20 ROM-Adresse - Ausgabe an RS232
         for(Tmp_a=0; Tmp_a<8; Tmp_a++) {    // For 0 To 7 Step +1
            RS232_Zahl(OW_Rom[Tmp_a]);       // ROM - Ausgabe an RS232
            if (Tmp_a < 7) { RS232_Put(Trennzeichen); }  // Trennzeichen an RS232 ausgeben
         }                                   // Ende FOR
         RS232_Zeilenumbruch();              // Zeilenumbruch
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende DS18B20 erkannt wurde
   }                                         // Ende SUCHE- erkannt wurde

   // --- SAVE- ------------------------------------------------------------------------
   else if (Str_vergleich(0,&Str3[0])) {     // wenn SAVE- erkannt wurde
      Tmp_b = naechste_Zahl(5);              // Zeiger auf erste Zahl einstellen
      Tmp_a = Zahl_erstellen(Tmp_b);         // erste Zahl einlesen
      Tmp_b = naechste_Zahl(Tmp_b);          // Zeiger auf nchste Zahl einstellen
      Tmp_c = Zahl_erstellen(Tmp_b);         // DS18B20-Daten einlesen
      // SAVE-TEMPSENSOR
      if (Str_vergleich(5,&Str4[0])) {       // wenn TEMPSENSOR erkannt wurde
         if ((Tmp_a >= 1) && (Tmp_a <= 8)) { // Sicherheitsabfrage
            if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
               eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][0], Tmp_c);  // ROM Adresse in EEprom schreiben
               Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
               Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
               if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                  eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][1], Tmp_c);  // ROM Adresse in EEprom schreiben
                  Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
                  Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
                  if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                     eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][2], Tmp_c);  // ROM Adresse in EEprom schreiben
                     Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
                     Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
                     if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                        eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][3], Tmp_c);  // ROM Adresse in EEprom schreiben
                        Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
                        Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
                        if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                           eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][4], Tmp_c);   // ROM Adresse in EEprom schreiben
                           Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
                           Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
                           if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                              eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][5], Tmp_c);  // ROM Adresse in EEprom schreiben
                              Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
                              Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
                              if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                                 eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][6], Tmp_c);  // ROM Adresse in EEprom schreiben
                                 Tmp_b = naechste_Zahl(Tmp_b);  // Zeiger auf nchste Zahl einstellen
                                 Tmp_c = Zahl_erstellen(Tmp_b);  // DS18B20-Daten einlesen
                                 if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
                                    eeprom_write_byte(&DS18B20_ROM_EEprom[Tmp_a-1][7], Tmp_c);  // ROM Adresse in EEprom schreiben
                                    OK_Flag = 1;  // Flag setzen
                                 }           // Ende Sicherheitsabfrage
                              }              // Ende Sicherheitsabfrage
                           }                 // Ende Sicherheitsabfrage
                        }                    // Ende Sicherheitsabfrage
                     }                       // Ende Sicherheitsabfrage
                  }                          // Ende Sicherheitsabfrage
               }                             // Ende Sicherheitsabfrage
            }                                // Ende Sicherheitsabfrage
         }                                   // Ende Sicherheitsabfrage
      }                                      // Ende TEMPSENSOR erkannt wurde
      // SAVE-MIN
      else if (Str_vergleich(5,&Str5[0])) {  // wenn MIN erkannt wurde
         if ((Tmp_a >= 1) && (Tmp_a <= 7)) { // Sicherheitsabfrage
            if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
               eeprom_write_byte(&MIN_Tacho_EEprom[Tmp_a-1], Tmp_c);  // MIN Tachosignal in EEprom schreiben
               OK_Flag = 1;                  // Flag setzen
            }                                // Ende Sicherheitsabfrage
         }                                   // Ende Sicherheitsabfrage
      }                                      // Ende MIN erkannt wurde
      // SAVE-PWM
      else if (Str_vergleich(5,&Str6[0])) {  // wenn PWM erkannt wurde
         if ((Tmp_a >= 1) && (Tmp_a <= 2)) { // Sicherheitsabfrage
            if ((Tmp_c >= 0) && (Tmp_c <= 255)) {  // Sicherheitsabfrage
               eeprom_write_byte(&PWM_Wert_EEprom[Tmp_a-1], Tmp_c);  // PWM-Wert in EEprom schreiben
               // PWM neu einstellen
               OCR0A = eeprom_read_byte(&PWM_Wert_EEprom[0]);  // PWM-Wert Kanal1 aus EEprom einlesen
               OCR0B = eeprom_read_byte(&PWM_Wert_EEprom[1]);  // PWM-Wert Kanal2 aus EEprom einlesen
               OK_Flag = 1;                  // Flag setzen
            }                                // Ende Sicherheitsabfrage
         }                                   // Ende Sicherheitsabfrage
      }                                      // Ende PWM erkannt wurde
      // SAVE-HAUPTALARM
      else if (Str_vergleich(5,&Str11[0])) { // wenn HAUPTALARM erkannt wurde
         Tmp_b = naechste_Zahl(15);          // Zeiger auf erste Zahl einstellen
         Tmp_c = Zahl_erstellen(Tmp_b);      // erste Zahl einlesen
         if ((Tmp_c >= 10) && (Tmp_c <= 3600)) {  // Sicherheitsabfrage
            union_var.un_Word = Tmp_c;       // TimeOut an Union bergeben
            eeprom_write_byte(&TimeOut_High_EEprom, union_var.un_Byte[1]);  // High-Byte Hauptalarm TimeOut in EEprom schreiben
            eeprom_write_byte(&TimeOut_Low_EEprom, union_var.un_Byte[0]);   // Low-Byte Hauptalarm TimeOut in EEprom schreiben
            OK_Flag = 1;                     // Flag setzen
         }                                   // Ende Sicherheitsabfrage
      }                                      // Ende HAUPTALARM erkannt wurde








   }                                         // Ende SAVE- erkannt wurde

   // --- INFO -------------------------------------------------------------------------
   else if (Str_vergleich(0,&Str7[0])) {     // wenn INFO erkannt wurde
      // DS18B20 Adressen anzeigen
      RS232_Zeilenumbruch();                 // Zeilenumbruch
      for (Tmp_a=0;Tmp_a<=7;Tmp_a++) {       // Temperatur 1 bis 8 ausgeben
         RS232_Array(&Text8[0]);             // Adresse DS18B20 Nr. - Ausgabe an RS232
         RS232_Zahl(Tmp_a+1);                // Nummer - Ausgabe an RS232
         RS232_Put(':');                     // Doppelpunkt an RS232 ausgeben
         RS232_Put(' ');                     // Leerzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][0])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][1])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][2])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][3])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][4])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][5])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][6])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Put(Trennzeichen);            // Trennzeichen an RS232 ausgeben
         RS232_Zahl(eeprom_read_byte(&DS18B20_ROM_EEprom[Tmp_a][7])); // ROM Adresse einlesen & an RS232 ausgeben
         RS232_Zeilenumbruch();              // Zeilenumbruch
      }                                      // Ende For
      // Tacho-MIN-Wert anzeigen
      RS232_Array(&Text9[0]);                // Tacho-MIN-Wert - Ausgabe an RS232
      for (Tmp_a=0;Tmp_a<=6;Tmp_a++) {       // Temperatur 1 bis 7 ausgeben
         RS232_Zahl(eeprom_read_byte(&MIN_Tacho_EEprom[Tmp_a])); // Tacho-MIN-Wert einlesen & an RS232 ausgeben
         if (Tmp_a < 6) { RS232_Put(Trennzeichen); }  // Trennzeichen an RS232 ausgeben
      }                                      // Ende For
      RS232_Zeilenumbruch();                 // Zeilenumbruch
      // PWM-Wert anzeigen
      RS232_Array(&Text10[0]);               // PWM-Wert - Ausgabe an RS232
      RS232_Zahl(eeprom_read_byte(&PWM_Wert_EEprom[0])); // PWM-Wert einlesen & an RS232 ausgeben
      RS232_Put(Trennzeichen);               // Trennzeichen an RS232 ausgeben
      RS232_Zahl(eeprom_read_byte(&PWM_Wert_EEprom[1])); // PWM-Wert einlesen & an RS232 ausgeben
      RS232_Zeilenumbruch();                 // Zeilenumbruch
      // TimeOut Hauptalarm starten
      RS232_Array(&Text12[0]);               // Hauptalarm TimeOut - Ausgabe an RS232
      union_var.un_Byte[1] = eeprom_read_byte(&TimeOut_High_EEprom);  // High-Byte Hauptalarm TimeOut aus EEprom einlesen
      union_var.un_Byte[0] = eeprom_read_byte(&TimeOut_Low_EEprom);   // Low-Byte Hauptalarm TimeOut aus EEprom einlesen
      RS232_Zahl(union_var.un_Word);         // TimeOut an RS232 ausgeben
      RS232_Zeilenumbruch();                 // Zeilenumbruch






   }                                         // Ende INFO erkannt wurde

   // --- VORALARM ---------------------------------------------------------------------
   else if (Str_vergleich(0,&Str8[0])) {     // wenn VORALARM erkannt wurde
      // VORALARM-ON
      if (Str_vergleich(9,&Str9[0])) {       // wenn ON erkannt wurde
         Setzen(Flag01,1);                   // Hand Voralarm aktivieren
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende ON erkannt wurde
      // VORALARM-OFF
      else if (Str_vergleich(9,&Str10[0])) { // wenn OFF erkannt wurde
         Loeschen(Flag01,1);                 // Reset - Hand Voralarm
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende OFF erkannt wurde
   }                                         // Ende VORALARM erkannt wurde

   // --- HAUPTALARM -------------------------------------------------------------------
   else if (Str_vergleich(0,&Str11[0])) {    // wenn HAUPTALARM erkannt wurde
      // HAUPTALARM-ON
      if (Str_vergleich(11,&Str9[0])) {      // wenn ON erkannt wurde
         Setzen(Flag01,2);                   // Hand Hauptalarm aktivieren
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende ON erkannt wurde
      // HAUPTALARM-OFF
      else if (Str_vergleich(11,&Str10[0])) { // wenn OFF erkannt wurde
         Loeschen(Flag01,2);                 // Reset - Hand Hauptalarm
         Hauptalarm_OFF();                   // Hauptalarm OFF
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende OFF erkannt wurde
   }                                         // Ende HAUPTALARM erkannt wurde





















   // --- RESET ------------------------------------------------------------------------
   else if (Str_vergleich(0,&Str12[0])) {    // wenn RESET erkannt wurde
      // RESET-HAUPTALARM
      if (Str_vergleich(6,&Str11[0])) {      // wenn HAUPTALARM erkannt wurde
         Setzen(Flag01,0);                   // Flag setzen - Hauptalarm zurcksetzen
         OK_Flag = 1;                        // Flag setzen
      }                                      // Ende HAUPTALARM erkannt wurde
      // RESET-SYSTEM
      if (Str_vergleich(6,&Str13[0])) {      // wenn SYSTEM erkannt wurde
         RS232_Array(&Text6[0]);             // OK - Ausgabe an RS232
         RS232_Zeilenumbruch();              // Zeilenumbruch
         Reset();
      }                                      // Ende SYSTEM erkannt wurde





   }                                         // Ende RESET erkannt wurde













   // OK an RS232 ausgeben
   if (OK_Flag > 0) {                        // wenn gesetzt, dann...
      RS232_Array(&Text6[0]);                // OK - Ausgabe an RS232
      RS232_Zeilenumbruch();                 // Zeilenumbruch
   }                                         // Ende wenn gesetzt
   // Reset Befehl
   Reset_Befehl();                           // Reset Befehlsstring
   // alte Tachodaten verwerfen und neu einlesen
   Tacho_Messung_starten();                  // 2Sek Verreigelung einstellen
}                                            // Ende RS232 Befehl ausfhren










// *************************************************************************************
// *   Hauptprogramm                                                                   *
// *************************************************************************************
int main(void) {                             // Beginn Hauptprogramm
   // --- Watchdog abschalten (nach einen Reset) ---------------------------------------
   MCUSR &= ~(1<<WDRF);                      // Watchdog OFF
   WDTCSR |= (1<<WDCE) | (1<<WDE);           // Watchdog OFF
   WDTCSR = 0x00;                            // Watchdog OFF

   // --- Definiton der Lokalen Variabeln ----------------------------------------------
//   uint8_t  Tmp01 = 0;                       // Temp Variable



   // --- Eingnge setzen --------------------------------------------------------------
   // PB0 - Kanal_1 Tachosignal_3
   // PB1 - Kanal_2 Tachosignal_1
   // PB2 - Kanal_2 Tachosignal_2
   // PB3 - MOSI - Kanal_2 Tachosignal_3
   // PB4 - MISO - frei
   // PB5 - SCK - frei
   DDRB &= ~((1<<DDB0) | (1<<DDB1) | (1<<DDB2) | (1<<DDB3) | (1<<DDB4) | (1<<DDB5));  // B-Register - als Eingang setzen
   PORTB |= (1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3) | (1<<PB4) | (1<<PB5);          // Interne Pull-Up fr alle setzen (1)
   // PC0 - AD-Wert Kanal_1
   // PC1 - AD-Wert Kanal_2
   // PC2 - DFM Tachosignal
   // PC3 - 1-Wire-Bus
   // PC4   Software I2C-Bus SCL (EDIT Hardware Pins vertauscht)
   // PC5   Software I2C-Bus SDA (EDIT Hardware Pins vertauscht)
   DDRC &= ~((1<<DDC0) | (1<<DDC1) | (1<<DDC2) | (1<<DDC3) | (1<<DDC4) | (1<<DDC5));  // C-Register - als Eingang setzen
   PORTC &= ~((1<<PC0) | (1<<PC1) | (1<<PC3) | (1<<PC4) | (1<<PC5));                  // Interne Pull-Up lschen (0)
   PORTC |= (1<<PC2);                        // Interne Pull-Up setzen (1)
   // PD4 - Kanal_1 Tachosignal_1
   // PD7 - Kanal_1 Tachosignal_2
   DDRD &= ~((1<<DDD4) | (1<<DDD7));         // D-Register - als Eingang setzen
   PORTD |= (1<<PD4) | (1<<PD7);             // Interne Pull-Up setzen (1)

   // --- Ausgnge setzen --------------------------------------------------------------
   // PD2 - Relais Voralarm
   // PD3 - Relais Hauptalarm
   // PD5 - PWM Kanal_2
   // PD6 - PWM Kanal_1
   DDRD |= (1<<DDD2) | (1<<DDD3) | (1<<DDD5) | (1<<DDD6);   // D-Register - als Ausgang setzen
   PORTD &= ~((1<<PD2) | (1<<PD3) | (1<<PD5) | (1<<PD6));   // Ports ausschalten (0)

   // --- Timer0 - PWM 36KHz -----------------------------------------------------------
   TCCR0A |= (1<<COM0A1) | (1<<COM0B1) | (1<<WGM00);  // non-inverting mode, Phase-Correct Mode1
   OCR0A = eeprom_read_byte(&PWM_Wert_EEprom[0]);  // PWM-Wert Kanal1 aus EEprom einlesen
   OCR0B = eeprom_read_byte(&PWM_Wert_EEprom[1]);  // PWM-Wert Kanal2 aus EEprom einlesen
   TCCR0B |= (1<<CS00);                      // Timer aktivieren, kein Vorteiler

   // --- Timer1 - 20ms Zeitbasis ------------------------------------------------------
   TIMSK1 |= (1<<TOIE1);                     // Timer1 Overflow Interrupt Enable
   TCNT1 = T1_vorladen;                      // Timer1 vorladen
   TCCR1B |= (1<<CS12) | (1<<CS10);          // Timer1 Vorteiler 1024 & aktivieren

   // --- RS232 initialisieren ---------------------------------------------------------
   // PD0   RS232 RXD (wird vom USI-Modul gesetzt)
   // PD1   RS232 TXD (wird vom USI-Modul gesetzt)
   RS232_Init();                             // RS232 initialisieren

   // --- 1-Wire Ports setzen ----------------------------------------------------------
   Init_1_Wire();                            // 1-Wire Ports einstellen

   // --- ADC aktivieren ---------------------------------------------------------------
   ADC_Init();                               // ADC aktivieren





   // --- Interrupt aktivieren ---------------------------------------------------------
   sei();                                    // Interrupts aktivieren

   // --- Initialisierung / Voreinstellung ---------------------------------------------


   // Reset an RS232 ausgeben
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   RS232_Array(&Text0[0]);                   // Reset - Ausgabe an RS232
   RS232_Zeilenumbruch();                    // Zeilenumbruch
   // Zeit-Zhler einstellen
   Tacho_Messung_starten();                  // 2Sek Verreigelung einstellen
   Zeit03 = Ein_verz;                        // 5Sek Einschaltverzgerung I2C-Bus Anzeige






   // --- Watchdog aktivieren ----------------------------------------------------------
   wdt_enable(WDTO_2S);                      // Watchdog auf 2 Sekunde einstellen


   // **********************************************************************************
   // *   Endlosschleife                                                               *
   // **********************************************************************************
   while(1) {                                // Beginn While - Endlosschleife

      // --- 2-Sekunden - Tacho-Messung ------------------------------------------------
      if (Zeit01 > 0) { Tacho_einlesen(); }  // wenn Messzeit, dann Tachosignale einlesen

      // --- 2-Sekunden - Auswertung ---------------------------------------------------
      else {                                 // wenn Messzeit abgelaufen, dann...

         // --- 1-Wire Temperaturmessung starten ---------------------------------------
         Temperatur_starten();               // Temperaturmessung starten (Dauer 800ms)

         // --- Verarbeitung wrend Temperaturmessung ----------------------------------
         AD_Wert_einlesen();                 // AD-Werte einlesen
         I2C_Daten_verarbeiten_senden();     // Daten verarbeiten und an I2C-Bus senden
         wdt_reset();                        // Watchdog Reset

         // --- 1-Wire Temperatur einlesen ---------------------------------------------
         Temperatur_einlesen();              // Temperatur einlesen


         // --- Daten & Fehler verarbeiten ---------------------------------------------













         // --- Daten an RS232 ausgeben ------------------------------------------------
         RS232_Daten_ausgeben();             // Daten an RS232 ausgeben

         // Ende
         Tacho_Messung_starten();            // 2Sek Verreigelung neu einstellen - Messung starten
      }                                      // Ende Messzeit abgelaufen








      // --- empfangene RS232 Daten auswerten ------------------------------------------
      if (Len_Puffer > 0) { RS232_einlesen(); }  // wenn RS232 Daten vorhanden - einlesen

      // --- I2C-Bus Reset bei Error ---------------------------------------------------
      if (TWSR == 0x00) {                    // wenn I2C-Bus Error erkannt, dann...
         TWCR |= (1<<TWSTO) | (1<<TWINT);    // TWSTO und TWINT setzen - TWINT lschen, weiter im I2C-Bus...
      }                                      // Ende I2C-Bus Error erkannt

      // --- Hardware Watchdog zurcksetzen --------------------------------------------
      wdt_reset();                           // Watchdog Reset

   }                                         // Ende While Schleife
}                                            // Ende Hauptprogramm



