// ***********************************************************
// Project:  RS232 Routinen
// Software: AVR-Studio v4.18 / WinAVR-20100110
// Author:   Jens Grtler
// Erstellt: 22.06.2014
// ***********************************************************


// --- einbinden von Libs ------------------------------------
#include <stdlib.h>                          // Umwandlung von Zahlen in Zeichenketten/Strings


// --- Definiton der Globalen Variabeln ----------------------
char Puffer[10];                             // RS232 Puffer fr Zahlen
   

// --- Baudrate festlegen ------------------------------------
//#define BAUD 300UL                           // 300 Baud
//#define BAUD 9600UL                        // 9600 Baud
#define BAUD 38400UL                       // 38400 Baud
// Berechnungen
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)  // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))  // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD)   // Fehler in Promille, 1000 = kein Fehler.
// Fehlerausgabe wenn Baudberechnung zu ungenau
#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))  // Fehlerausgabe
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch!  // Fehlerausgabe
#endif                                       // Fehlerausgabe


// *************************************************************************************
// *   FUNKTION - RS232 Init                                                           *
// *************************************************************************************
void RS232_Init (void) {                     // Beginn Funktion RS232-Init
   // Ports als Eingnge setzen
   DDRD &= ~((1<<DDD0) | (1<<DDD1));         // D-Register - PD0 und PD1 als Eingang setzen
   PORTD |= (1<<PD0) | (1<<PD1);             // Interne Pull-Up fr alle setzen (1)
   // RS232 aktivieren
   UCSR0B |= (1<<RXEN0) | (1<<TXEN0) | (1<<RXCIE0);  // UART RX und TX aktivieren, RX Interrupt-Vektor aktivieren
   UCSR0C |= (1 << UCSZ01) | (1 << UCSZ00);  // RS232 Type - Asynchron 8N1
   // Baudrate festlegen
   UBRR0H = UBRR_VAL >> 8;                   // Baudrate High-Register bergeben
   UBRR0L = UBRR_VAL & 0xFF;                 // Baudrate Low-Register bergeben
}                                            // Ende Funktion RS232-Init


// *************************************************************************************
// *   FUNKTION - Put   --->   ein Zeichen ber RS232 senden                           *
// *************************************************************************************
void RS232_Put (uint8_t Tmp) {               // Beginn Funktion Put
   while (!(UCSR0A & (1<<UDRE0))) {          // warten bis Senden mglich ist
      asm volatile("NOP");                   // nichts tun
   }
   UDR0 = Tmp;                               // bergabe Wert an RS232 ausgeben
}                                            // Ende Funktion Put


// *************************************************************************************
// *   FUNKTION - Zeilenumbruch   --->   Zeilenumbruch ber RS232 senden               *
// *************************************************************************************
void RS232_Zeilenumbruch (void) {            // Beginn Funktion RS232-Zeilenumbruch
   RS232_Put(13);                            // CR Senden
   RS232_Put(10);                            // LF Senden
}                                            // Ende Funktion RS232-Zeilenumbruch


// *************************************************************************************
// *   FUNKTION - Zahlen   --->   Variabeln als Zahlen ber RS232 senden               *
// *   Info: http://www.mikrocontroller.net/articles/FAQ#itoa.28.29                    *
// *************************************************************************************
void RS232_Zahl (int32_t Tmp) {              // Beginn Funktion Zahlen
   // Nagative Zahlen wandeln
   if (Tmp < 0) {                            // wenn Negative Zahl, dann...
      RS232_Put('-');                        // Minuszeichen an RS232 ausgeben
      Tmp = 0 - Tmp;                         // in positive Zahl wandeln
   }                                         // Ende wenn Negative Zahl
   // Zahlen ausgeben
   char *s;                                  // String ???
   utoa(Tmp, Puffer, 10);                    // Variable in String umwandeln
   s = Puffer;                               // Puffer an String bergeben
   while (*s) {                              // wiederholen bis String vollstndig bertragen
      // so lange *s != '\0' also ungleich dem "String-Endezeichen"
      RS232_Put(*s);                         // ein Zeichen ber RS232 ausgeben
      s++;                                   // s = s + 1
   }                                         // Ende while
}                                            // Ende Funktion Zahlen


/*
// *************************************************************************************
// *   FUNKTION - String   --->   String ber RS232 senden                             *
// *************************************************************************************
void RS232_String (char *s) {                // Beginn Funktion String
   while (*s) {                              // wiederholen bis String vollstndig bertragen
      // so lange *s != '\0' also ungleich dem "String-Endezeichen"
      RS232_Put(*s);                         // ein Zeichen ber RS232 ausgeben
      s++;                                   // s = s + 1
   }                                         // Ende while
}                                            // Ende Funktion String
*/


// *************************************************************************************
// *   FUNKTION - Flash-Array   --->   String ber RS232 senden                        *
// *************************************************************************************
void RS232_Array(const unsigned char *text) {  // Beginn Funktion String
   unsigned char Zeichen = 1;                // Voreinstellung While - Zeichen
   while (Zeichen)                       {   // wiederholen bis String vollstndig bertragen
      Zeichen = pgm_read_byte(text);         // wiederholen bis String vollstndig bertragen
      RS232_Put(Zeichen);                    // ein Zeichen ber RS232 ausgeben
      text++;                                // nchstes Zeichen
   }
}                                            // Ende Funktion String









