// ***********************************************************
// Project:  Software I2C-Bus - Master
// Software: AVR-Studio v4.18 / WinAVR-20090313
// Author:   Jens Grtler
// Erstellt: 14.10.2011
// ***********************************************************


// --- einbinden von Libs ------------------------------------
#include <util/delay.h>                      // Zeitschleife einbinden


// *************************************************************************************
// *   FUNKTION - Pausenzeiten   --->   1000 = 1 Sekunde                               *
// *************************************************************************************
//void Pause (uint16_t a) {                    // Beginn Funktion Pause
//   for(; a>0; a--) {                         // For a=Zahl To 0 Step -1
//      _delay_ms(1);                          // 1ms Pause
//   }                                         // Ende FOR
//}                                            // Ende Funktion Pause


// --- Definiton der Konstanten ------------------------------
#define SCL_DDR          DDRC                // DDR Register fr SCL
#define SCL_PORT         PORTC               // Port Register fr SCL
#define SCL_PIN          4                   // Pin fr SCL

#define SDA_DDR          DDRC                // DDR Register fr SDA
#define SDA_PORT         PORTC               // Port Register fr SDA
#define SDA_PIN          5                   // Pin fr SDA
#define SDA_PIN_reg      PINC                // Pin Register fr SDA

#define Zeit             8                   // Zeit fr _delay_us() - I2C Geschwindigkeit





// *************************************************************************************
// *   ROUTINE - SCL ON (High = als Eingang schalten)                                  *
// *************************************************************************************
void SCL_ON(void) {                          // Beginn SCL ON
   SCL_DDR &= ~(1<<SCL_PIN);                 // SCL als Eingang setzen
   SCL_PORT |= (1<<SCL_PIN);                 // Internen Pull-Up setzen (1)
}                                            // Ende SCL ON


// *************************************************************************************
// *   ROUTINE - SCL OFF (Low = als Ausgang schalten)                                  *
// *************************************************************************************
void SCL_OFF(void) {                         // Beginn SCL OFF
   SCL_DDR |= (1<<SCL_PIN);                  // SCL als Ausgang setzen
   SCL_PORT &= ~(1<<SCL_PIN);                // Ports ausschalten (0)
}                                            // Ende SCL OFF


// *************************************************************************************
// *   ROUTINE - SDA ON (High = als Eingang schalten)                                  *
// *************************************************************************************
void SDA_ON(void) {                          // Beginn SDA ON
   SDA_DDR &= ~(1<<SDA_PIN);                 // SDA als Eingang setzen
   SDA_PORT |= (1<<SDA_PIN);                 // Internen Pull-Up setzen (1)
}                                            // Ende SDA ON


// *************************************************************************************
// *   ROUTINE - SDA OFF (Low = als Ausgang schalten)                                  *
// *************************************************************************************
void SDA_OFF(void) {                         // Beginn SDA OFF
   SDA_DDR |= (1<<SDA_PIN);                  // SDA als Ausgang setzen
   SDA_PORT &= ~(1<<SDA_PIN);                // Ports ausschalten (0)
}                                            // Ende SDA OFF


// *************************************************************************************
// *   ROUTINE - I2C-Write                                                             *
// *************************************************************************************
uint8_t Soft_I2C_Write(uint8_t Tmp_Dat) {    // Beginn I2C-Write
   uint8_t Tmp_ACK = 0;                      // ACK vom Slave (1 = ACK OK)
   // 8 Bits senden
   int a = 1;                                // For initialisieren
   for(; a<=8; a++) {                        // For a=1 To 8 Step 1
      SDA_OFF();                             // SDA Low  - Reset
      if (Tmp_Dat & (1<<7)) { SDA_ON(); }    // SDA High, wenn Bit7 = 1
      SCL_ON();                              // SCL High - Bit senden
      _delay_us(Zeit);                       // Zeit s warten
      SCL_OFF();                             // SCL Low  - Bit senden beebdet
      _delay_us(Zeit);                       // Zeit s warten
      Tmp_Dat = (Tmp_Dat<<1);                // SHL 1
   }                                         // Ende FOR
   // 9-tes Bit empfangen
   SDA_ON();                                 // deact SDA (als Eingang setzen)
   SCL_ON();                                 // SCL High - Bit empfangen
   _delay_us(Zeit);                          // Zeit s warten
   if (!(SDA_PIN_reg & (1<<SDA_PIN))) { Tmp_ACK = 1; }  // wenn SDA Low, dann ACK empfangen
   SCL_OFF();                                // SCL Low  - Ende Bit empfangen
   _delay_us(Zeit);                          // Zeit s warten
   // ACK bergeben
   return Tmp_ACK;                           // ACK bergeben
}                                            // Ende I2C-Write


// *************************************************************************************
// *   ROUTINE - I2C-Read (0 = NACK)                                                   *
// *************************************************************************************
uint8_t Soft_I2C_Read(uint8_t Tmp_Ack) {     // Beginn I2C-Read
   uint8_t Tmp_Data = 0;                     // Daten vom Slave
   // 8 Bits empfangen
   SDA_ON();                                 // deact SDA (als Eingang setzen)
   int a = 1;                                // For initialisieren
   for(; a<=8; a++) {                        // For a=1 To 8 Step 1
      Tmp_Data = (Tmp_Data<<1);              // SHL 1
      SCL_ON();                              // SCL High - Bit empfangen
      _delay_us(Zeit);                       // Zeit s warten
      if (SDA_PIN_reg & (1<<SDA_PIN)) { Tmp_Data++; }  // wenn SDA High, dann Daten +1
      SCL_OFF();                             // SCL Low  - Ende Bit empfangen
      _delay_us(Zeit);                       // Zeit s warten
   }                                         // Ende FOR
   // 9-tes Bit senden
   SDA_ON();                                 // SDA High - NACK Voreinstellung
   if (Tmp_Ack > 0) { SDA_OFF(); }           // SDA Low  - ACK einstellen
   SCL_ON();                                 // SCL High - Bit senden
   _delay_us(Zeit);                          // Zeit s warten
   SCL_OFF();                                // SCL Low  - Ende Bit senden
   _delay_us(Zeit);                          // Zeit s warten
   // Daten bergeben
   return Tmp_Data;                          // Daten bergeben
}                                            // Ende I2C-Read








/*
#I2C_read                               'lesen + Ack ohne Auswertung
gosub read_sub                          'Daten zuweisen
SDA = off                               'SDA Low und Pulse
gosub I2C_Pulse                         'SDA Low und Pulse
return
'---------------------------------------------------------------------------------------
#I2C_read_last                          'lesen beenden + No Ack ohne Auswertung
gosub read_sub                          'Daten zuweisen
SDA = on                                'SDA High und Pulse
gosub I2C_Pulse                         'SDA High und Pulse
return
'---------------------------------------------------------------------------------------
#read_sub                               '8 Bit lesen
I2C_tmp = 0                             'Reset Daten
deact SDA                               'Datenleitung = Eingang
for sv1 = 1 to 8                        'Bit fr Bit einlesen
I2C_tmp = I2C_tmp shl 1                 '1 Bit nach links
SCL = on                                'Start lesen
if SDA then I2C_tmp = I2C_tmp + 1       'Datenzuweisung
SCL = off                               'Stop lesen
next
return
'---------------------------------------------------------------------------------------
#I2C_Pulse                              'Pulse SCL
tog SCL : tog SCL                       'Pulse SCL
return
'---------------------------------------------------------------------------------------
*/







// *************************************************************************************
// *   ROUTINE - I2C-Start                                                             *
// *************************************************************************************
uint8_t Soft_I2C_Start(uint8_t Tmp_Adr) {    // Beginn I2C-Start
   SDA_ON();                                 // SDA High - Re-Start Bedingung
   _delay_us(Zeit);                          // Zeit s warten
   SCL_ON();                                 // SCL High - Re-Start Bedingung
   _delay_us(Zeit);                          // Zeit s warten
   SDA_OFF();                                // SDA Low  - Start Bedingung
   _delay_us(Zeit);                          // Zeit s warten
   SCL_OFF();                                // SCL Low  - Start Bedingung
   _delay_us(Zeit);                          // Zeit s warten
   return Soft_I2C_Write(Tmp_Adr);           // Adresse senden
}                                            // Ende I2C-Start


// *************************************************************************************
// *   ROUTINE - I2C-Stop                                                              *
// *************************************************************************************
void Soft_I2C_Stop(void) {                   // Beginn I2C-Stop
   SDA_OFF();                                // SDA Low  - Stop Bedingung
   _delay_us(Zeit);                          // Zeit s warten
   SCL_ON();                                 // SCL High - Stop Bedingung
   _delay_us(Zeit);                          // Zeit s warten
   SDA_ON();                                 // SDA High - Stop Bedingung
   _delay_us(Zeit);                          // Zeit s warten
}                                            // Ende I2C-Stop







